package com.chelvc.cloud.vehicle.client;


import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.MerchantCartItemDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsCartItemDTO;
import com.chelvc.cloud.vehicle.client.param.OmsCartItemModifyParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 购物车Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@FeignClient("vehicle")
public interface OmsCartItemClient {

    /**
     * 查询购物车中是否包含该商品，有增加数量，无添加到购物车
     */
    @PostMapping("/omsCartItem/add")
    int add(@RequestBody OmsCartItemModifyParam cartItem);

    /**
     * 根据会员编号获取购物车列表
     */
    @GetMapping("/omsCartItem/listCartItem")
    List<OmsCartItemDTO> listCartItem();


    /**
     * 修改某个购物车商品的数量
     */
    @PostMapping("/omsCartItem/updateQuantity")
    int updateQuantity(@RequestParam("id") Long id, @RequestParam("quantity") Integer quantity);

    /**
     * 批量删除购物车中的商品
     */
    @DeleteMapping("/omsCartItem")
    void delete(@RequestParam("ids") List<Long> ids);

    /**
     * 修改购物车中商品的规格
     */
    @PostMapping("/omsCartItem/updateAttr")
    int updateAttr(@RequestBody OmsCartItemModifyParam cartItem);

    /**
     * 清空购物车
     */
    @PostMapping("/omsCartItem/clear")
    void clear();

    /**
     * 根据会员编号及商家获取购物车列表
     */
    @GetMapping("/omsCartItem/listMerchantCartItem")
    MerchantCartItemDTO listMerchantCartItem(@RequestParam("merchantId") Long merchantId);
}
