package com.chelvc.cloud.vehicle.client;

import com.chelvc.cloud.vehicle.client.dto.ConfirmOrderResultDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.client.dto.WechatUnifiedOrder;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderPagingParam;
import com.chelvc.cloud.vehicle.client.param.OrderRefundParam;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 订单Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@FeignClient("vehicle")
public interface OmsOrderClient {

    /**
     * 根据用户购物车信息生成确认单信息
     *
     * @param orderParam
     */
    @PostMapping("/omsOrder/generateConfirmOrder")
    ConfirmOrderResultDTO generateConfirmOrder(@RequestBody OmsOrderModifyParam orderParam);

    /**
     * 根据提交信息生成订单
     */
    @PostMapping("/omsOrder/generateOrder")
    WechatUnifiedOrder generateOrder(@RequestBody OmsOrderModifyParam orderParam);

    /**
     * 支付成功后的回调
     */
    @PostMapping("/omsOrder/paySuccess")
    Integer paySuccess(@RequestParam("orderId") Long orderId, @RequestParam("payType") Integer payType);

    /**
     * 自动取消超时订单
     */
    @PostMapping("/omsOrder/cancelTimeOutOrder")
    Integer cancelTimeOutOrder();

    /**
     * 取消单个超时订单
     */
    @PostMapping("/omsOrder/cancelOrder")
    void cancelOrder(@RequestParam("orderId") Long orderId);

    /**
     * 发送延迟消息取消订单
     */
    @PostMapping("/omsOrder/sendDelayMessageCancelOrder")
    void sendDelayMessageCancelOrder(@RequestParam("orderId") Long orderId);

    /**
     * 确认收货
     */
    @PostMapping("/omsOrder/confirmReceiveOrder")
    void confirmReceiveOrder(@RequestParam("orderId") Long orderId);

    /**
     * 查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getOrderPaging")
    Pagination<OmsOrderDTO> getOrderPaging(@SpringQueryMap OrderPagingParam param);

    /**
     * 分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getOmsOrderPaging")
    Pagination<OmsOrderDTO> getOmsOrderPaging(@SpringQueryMap OrderPagingParam param);

    /**
     * 根据订单ID获取订单详情
     */
    @GetMapping("/omsOrder/detail")
    OmsOrderDetailDTO detail(@RequestParam("orderId") Long orderId);

    /**
     * 用户根据订单ID删除订单
     */
    @DeleteMapping("/omsOrder")
    void deleteOrder(@RequestParam("orderId") Long orderId);

    /**
     * 获取订单量及收入
     *
     * @return
     */
    @GetMapping("/omsOrder/getOrderQuantity")
    Map<String, Object> getOrderQuantity(@RequestParam("merchantId") Long merchantId);

    /**
     * 商家端分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getMerchantOrderPaging")
    Pagination<OmsOrderDTO> getMerchantOrderPaging(@SpringQueryMap OrderPagingParam param);

    /**
     * 商家订单核验
     *
     * @param orderId
     * @param merchantId
     */
    @PostMapping("/omsOrder/orderVerification")
    void orderVerification(@RequestParam("orderId") Long orderId, @RequestParam("merchantId") Long merchantId);

    /**
     * 根据订单核销码获取订单详情
     */
    @GetMapping("/omsOrder/getOrderDetail")
    OmsOrderDetailDTO getOrderDetail(@RequestParam("orderSn") String orderSn,@RequestParam("merchantId") Long merchantId) ;

    /**
     * 订单支付回调
     */
    @PostMapping("/order/notify")
    void orderNotify(@RequestBody SxyNotifyParam param);

    /**
     * 退款申请
     * @param param 查询参数
     * @return
     */
    @PostMapping("/order/refund")
    void orderRefund(@RequestBody OrderRefundParam param);

    /**
     * 退款申请回调
     */
    @PostMapping("/order/refund/notify")
    void orderRefundNotify(@RequestBody SxyNotifyParam param);
}
