package com.chelvc.cloud.vehicle.server.controller;

import com.chelvc.cloud.vehicle.client.constant.ExamineType;
import com.chelvc.cloud.vehicle.client.dto.DynamicCommentDTO;
import com.chelvc.cloud.vehicle.client.param.AddDynamicCommentParam;
import com.chelvc.cloud.vehicle.client.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.client.param.QueryDynamicCommentParam;
import com.chelvc.cloud.vehicle.server.service.DynamicCommentService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 动态评论Service接口
 *
 * @author igl
 * @date 2023-02-25
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DynamicCommentController {

    private final DynamicCommentService dynamicCommentService;

    /**
     * 查询动态评论列表
     */
    @GetMapping("/dynamicComment/queryPageList")
    public Pagination<DynamicCommentDTO> queryPageList(@SpringQueryMap QueryDynamicCommentParam param,
                                                       @RequestParam("pageNum") Integer pageNum,
                                                       @RequestParam("pageSize") Integer pageSize) {
        return dynamicCommentService.queryPageList(param, pageNum, pageSize);
    }

    /**
     * 查询动态信息列表
     */
    @GetMapping("/dynamicComment/queryAdminPageList")
    public Pagination<DynamicCommentDTO> queryAdminPageList(QueryAdminDynamicParam param) {
        return dynamicCommentService.queryAdminPageList(param);
    }

    /**
     * 修改动态评论
     */
    @PostMapping("/dynamicComment/insertByBo")
    public Long insertByBo(@RequestBody AddDynamicCommentParam param, @RequestParam("userId") Long userId) {
        return dynamicCommentService.insertByBo(param, userId);
    }

    /**
     * 给评论添加点赞/踩
     *
     * @param dynamicStatus
     * @param contentId
     */
    @PostMapping("/dynamicComment/addNum")
    public Long addNum(@RequestParam("dynamicStatus") Integer dynamicStatus,
                       @RequestParam("contentId") Long contentId) {
        return dynamicCommentService.addNum(dynamicStatus, contentId);
    }

    /**
     * 取消评论点赞/踩
     *
     * @param dynamicStatus
     * @param contentId
     */
    @PostMapping("/dynamicComment/reduceNum")
    public Long reduceNum(@RequestParam("dynamicStatus") Integer dynamicStatus,
                          @RequestParam("contentId") Long contentId) {
        return dynamicCommentService.reduceNum(dynamicStatus, contentId);
    }

    /**
     * 评论手动审核
     *
     * @param id
     * @param examine
     * @author igl
     * @date 2023/8/17 17:25
     */
    @PostMapping("/dynamicComment/examine")
    public void examine(@RequestParam("id") Long id, @RequestParam("examine") ExamineType examine) {
        dynamicCommentService.examine(id, examine);
    }

    /**
     * 自动审核
     *
     * @param id             评论标识
     * @param platformStatus 平台状态
     * @param examineResult  审核结果
     * @author igl
     * @date 2023/9/12 17:25
     */
    @PostMapping("/dynamicComment/autoAudit")
    public void autoAudit(@RequestParam("id") Long id, @RequestParam("platformStatus") int platformStatus,
                          @RequestParam("examineResult") int examineResult) {
        dynamicCommentService.autoAudit(id, platformStatus, examineResult);
    }
}
