package com.chelvc.cloud.vehicle.server.controller;

import java.util.Map;

import com.chelvc.cloud.vehicle.client.dto.ConfirmOrderResultDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.client.dto.WechatUnifiedOrder;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderPagingParam;
import com.chelvc.cloud.vehicle.client.param.OrderRefundParam;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.server.service.OmsOrderService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 订单Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderController {
    private final OmsOrderService omsOrderService;

    /**
     * 根据用户购物车信息生成确认单信息
     *
     * @param orderParam
     */
    @PostMapping("/omsOrder/generateConfirmOrder")
    public ConfirmOrderResultDTO generateConfirmOrder(@RequestBody OmsOrderModifyParam orderParam) {
        return omsOrderService.generateConfirmOrder(orderParam);
    }

    /**
     * 根据提交信息生成订单
     */
    @PostMapping("/omsOrder/generateOrder")
    public WechatUnifiedOrder generateOrder(@RequestBody OmsOrderModifyParam orderParam) {
        return omsOrderService.generateOrder(orderParam);
    }

    /**
     * 支付成功后的回调
     */
    @PostMapping("/omsOrder/paySuccess")
    public Integer paySuccess(@RequestParam("orderId") Long orderId, @RequestParam("payType") Integer payType) {
        return omsOrderService.paySuccess(orderId, payType);
    }

    /**
     * 自动取消超时订单
     */
    @PostMapping("/omsOrder/cancelTimeOutOrder")
    public Integer cancelTimeOutOrder() {
        return omsOrderService.cancelTimeOutOrder();
    }

    /**
     * 取消单个超时订单
     */
    @PostMapping("/omsOrder/cancelOrder")
    public void cancelOrder(@RequestParam("orderId") Long orderId) {
        omsOrderService.cancelOrder(orderId);
    }

    /**
     * 发送延迟消息取消订单
     */
    @PostMapping("/omsOrder/sendDelayMessageCancelOrder")
    public void sendDelayMessageCancelOrder(@RequestParam("orderId") Long orderId) {
        omsOrderService.sendDelayMessageCancelOrder(orderId);
    }

    /**
     * 确认收货
     */
    @PostMapping("/omsOrder/confirmReceiveOrder")
    public void confirmReceiveOrder(@RequestParam("orderId") Long orderId) {
        omsOrderService.confirmReceiveOrder(orderId);
    }

    /**
     * 查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getOrderPaging")
    public Pagination<OmsOrderDTO> getOrderPaging(OrderPagingParam param) {
        return omsOrderService.getOrderPaging(param);
    }

    /**
     * 分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getOmsOrderPaging")
    public Pagination<OmsOrderDTO> getOmsOrderPaging(OrderPagingParam param) {
        return omsOrderService.getOmsOrderPaging(param);
    }

    /**
     * 根据订单ID获取订单详情
     */
    @GetMapping("/omsOrder/detail")
    public OmsOrderDetailDTO detail(@RequestParam("orderId") Long orderId) {
        return omsOrderService.detail(orderId);
    }

    /**
     * 用户根据订单ID删除订单
     */
    @DeleteMapping("/omsOrder")
    public void deleteOrder(@RequestParam("orderId") Long orderId) {
        omsOrderService.deleteOrder(orderId);
    }

    /**
     * 获取订单量及收入
     *
     * @return
     */
    @GetMapping("/omsOrder/getOrderQuantity")
    public Map<String, Object> getOrderQuantity(@RequestParam("merchantId") Long merchantId) {
        return omsOrderService.getOrderQuantity(merchantId);
    }

    /**
     * 商家端分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/omsOrder/getMerchantOrderPaging")
    public Pagination<OmsOrderDTO> getMerchantOrderPaging(OrderPagingParam param) {
        return omsOrderService.getMerchantOrderPaging(param);
    }

    /**
     * 商家订单核验
     *
     * @param orderId
     * @param merchantId
     */
    @PostMapping("/omsOrder/orderVerification")
    public void orderVerification(@RequestParam("orderId") Long orderId, @RequestParam("merchantId") Long merchantId) {
        omsOrderService.orderVerification(orderId, merchantId);
    }

    /**
     * 根据订单核销码获取订单详情
     */
    @GetMapping("/omsOrder/getOrderDetail")
    public OmsOrderDetailDTO getOrderDetail(@RequestParam("orderSn") String orderSn,@RequestParam("merchantId") Long merchantId) {
        return omsOrderService.getOraderDetail(orderSn,merchantId);
    }

    /**
     * 订单支付回调
     */
    @PostMapping("/order/notify")
    public void orderNotify(@RequestBody SxyNotifyParam param){
        omsOrderService.orderNotify(param);
    }

    /**
     * 退款申请
     * @param param 参数
     * @return
     */
    @PostMapping("/order/refund")
    public void orderRefund(@RequestBody OrderRefundParam param) {
        omsOrderService.orderRefund(param);
    }

    /**
     * 退款申请回调
     */
    @PostMapping("/order/refund/notify")
    public void orderRefundNotify(@RequestBody SxyNotifyParam param){
        omsOrderService.orderRefundNotify(param);
    }
}
