package com.chelvc.cloud.vehicle.server.controller;


import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.dto.OrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyPagingParam;
import com.chelvc.cloud.vehicle.server.service.OmsOrderReturnApplyService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 订单退货申请接口
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderReturnApplyController {

    private final OmsOrderReturnApplyService omsOrderReturnApplyService;

    @GetMapping("/omsOrderReturnApply/getOrderReturnApplyInfo")
    public OmsOrderReturnApplyDTO getOrderReturnApplyInfo(@RequestParam("id") Long id) {
        return omsOrderReturnApplyService.getOrderReturnApplyInfo(id);
    }

    /**
     * 新增订单退货申请
     * @param param
     * @return
     */
    @PostMapping("/omsOrderReturnApply/add")
    public Long add(@SpringQueryMap OrderReturnApplyModifyParam param) {
        return omsOrderReturnApplyService.insertOmsOrderReturnApply(param);
    }

    /**
     * 修改订单退货申请
     * @param param
     * @return
     */
    @PutMapping("/omsOrderReturnApply/update")
    public Long update(@RequestParam("id") Long id,@SpringQueryMap OrderReturnApplyModifyParam param) {
        return omsOrderReturnApplyService.updateOmsOrderReturnApply(id,param);
    }

    /**
     * 订单退货申请分页查询
     * @param param
     * @return
     */
    @GetMapping("/omsOrderReturnApply/paging")
    public Pagination<OrderReturnApplyDTO> paging(@SpringQueryMap OrderReturnApplyPagingParam param) {
        return omsOrderReturnApplyService.getOrderReturnApplyPaging(param);
    }



    /**
     * 订单退货申请查询详情
     * @param id
     * @return
     */
    @GetMapping("/omsOrderReturnApply/getDetail")
    public OrderReturnApplyDTO getDetail(@RequestParam("id")Long id) {
        return omsOrderReturnApplyService.selectOmsOrderReturnApplyById(id);
    }

    /**
     * 删除订单退货申请
     * @param id
     * @return
     */
    @GetMapping("/omsOrderReturnApply/delete")
    public void delete(@RequestParam("id")Long id) {
        omsOrderReturnApplyService.deleteOmsOrderReturnApplyById(id);
    }

}
