package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.client.dto.MerchantAssetDTO;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.client.param.TransferNotifyParam;
import com.chelvc.cloud.vehicle.client.param.WithdrawNotifyParam;
import com.chelvc.cloud.vehicle.client.param.WithdrawParam;
import com.chelvc.cloud.vehicle.server.entity.Asset;

import java.math.BigDecimal;

public interface AssetService {
    /**
     * 查看资产
     * @param userId 用户标识
     * @param type 类型：1-用户；2-商家
     * @return
     */
    Asset queryAsset(Long userId, Integer type);

    /**
     * 更新资产
     * @param operateType 操作类型：0-增加；1-减少
     * @param operateAmount 操作金额
     * @param userId 用户标识
     * @param type 类型：0-平台；1-用户；2-商家
     * @param amountType 金额类型：1-商户资产；2-冻结金额
     * @return
     */
    BigDecimal updateAsset(Integer operateType, BigDecimal operateAmount, Long userId, Integer type, Integer amountType);

    void updatePlatformAsset(BigDecimal platformRealityAmount, BigDecimal payAmount);

    MerchantAssetDTO getMerchantAsset(Long merchantId);


    String withdrawNotify(SxyNotifyParam param);

    String transferNotify(SxyNotifyParam param);
    void withdraw(Long userId, Integer type, BigDecimal amount);

    void transfer(Long sourceUserId, Long targetUserId, Integer type, BigDecimal amount, String remark);

    /**
     *
     * @param userId 用户/商户id； 0-平台
     * @param type 类型：0-平台；1-用户；2-商户
     * @param operate 0-增加；1-减少
     * @param amount 金额
     */
    int updateAmount(Long userId, Integer type, Integer operate, BigDecimal amount);

    void withdrawApply(WithdrawParam param);

    void reduceFrozenAsset(BigDecimal payAmount, Long userId, Integer type);

    /**
     * 冻结金额
     * @param amount
     * @param userId
     * @param type
     * @return
     */
    int frozenAmount(BigDecimal amount, Long userId, Integer type);

    BigDecimal queryTotal(Long userId, Integer type);
}
