package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.QueryMerchantBalanceParam;
import com.chelvc.cloud.vehicle.server.entity.BalanceDetail;
import com.chelvc.framework.common.model.Pagination;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

public interface BalanceDetailService {
    /**
     * 记录余额明细
     * @param orderId 订单id
     * @param type 类型：0-平台；1-用户；2-商户
     * @param userId 用户id
     * @param operateAmount 操作金额
     * @param deduct 扣除金额：提现-手续费；分佣扣税
     * @param realityAmount 实际金额
     * @param surplusAmount 当前余额
     * @param flowType 流水类型：0-收入；1-支出
     * @param operateType 操作类型：0-商品出售；1-用户分佣；2-商户分佣；3-平台分佣；4-提现；5-用户支付；6-用户核销；7-转账；8-退款；9-平台优惠券补贴
     * @param sourceId 来源id
     */
    void recordFlow(Long orderId, Integer type,  Long userId, BigDecimal operateAmount, BigDecimal deduct,
                    BigDecimal realityAmount, BigDecimal surplusAmount,
                    Integer flowType, Integer operateType, Long sourceId);

    Pagination<BalanceDetailDTO> queryByMerchantId(QueryMerchantBalanceParam param, Integer pageNum,
                                                   Integer pageSize, Integer type);

    WalletDTO getMerchantAsset(@Param("merchantId") Long merchantId);

    /**
     * 收益流水记录
     * @return 收益流水记录列表
     */
    List<BalanceRecordDTO> listBalanceRecord(Integer type, Long date);

    /**
     * 获取用户得总收益、本周收益、本月收益、可提现金额
     * @return
     */
    EarningsDTO getBalanceRecord(Integer type);

    /**
     *
     * @param orderId 订单id
     * @param type 类型：0-平台；1-用户；2-商户
     * @param userId 用户id
     * @param flowType 流水类型：0-收入；1-支出
     * @param operateType 操作类型：0-商品出售；1-用户分佣；2-商户分佣；3-平台分佣；4-提现；5-用户支付；6-用户核销；7-转账；8-退款；9-平台优惠券补贴
     */
    BalanceDetail queryBy(Long orderId, Integer type, Long userId, Integer flowType, Integer operateType);
}
