package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderPagingParam;
import com.chelvc.cloud.vehicle.client.param.OrderRefundParam;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.database.support.EnhanceService;
import com.upay.sdk.onlinepay.builder.OrderResponse;
import org.springframework.web.bind.annotation.RequestBody;

import java.math.BigDecimal;
import java.util.Map;

/**
 * 订单Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
public interface OmsOrderService extends EnhanceService<OmsOrder>
{
    /**
     * 查询商家上个月的营业额
     * @param merchantId
     * @return
     */
    BigDecimal queryAmountByLastMonth(Long merchantId);

    /**
     * 根据用户购物车信息生成确认单信息
     * @param orderParam
     */
    ConfirmOrderResultDTO generateConfirmOrder(OmsOrderModifyParam orderParam);

    /**
     * 根据提交信息生成订单
     */
    WechatUnifiedOrder generateOrder(OmsOrderModifyParam orderParam);

    /**
     * 支付成功后的回调
     */
    Integer paySuccess(Long orderId, Integer payType);

    /**
     * 自动取消超时订单
     */
    Integer cancelTimeOutOrder();

    /**
     * 取消单个超时订单
     */
    void cancelOrder(Long orderId);

    /**
     * 修改订单状态
     */
    void updateOrderStatus(Long orderId,Integer status);

    /**
     * 发送延迟消息取消订单
     */
    void sendDelayMessageCancelOrder(Long orderId);

    /**
     * 确认收货
     */
    void confirmReceiveOrder(Long orderId);

    /**
     * 查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    Pagination<OmsOrderDTO> getOrderPaging(OrderPagingParam param);

    /**
     * 分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    Pagination<OmsOrderDTO> getOmsOrderPaging(OrderPagingParam param);

    /**
     * 根据订单ID获取订单详情
     */
    OmsOrderDetailDTO detail(Long orderId);

    /**
     * 用户根据订单ID删除订单
     */
    void deleteOrder(Long orderId);

    /**
     * 获取订单量及收入
     * @return
     */
    Map<String,Object> getOrderQuantity(Long merchantId);

    /**
     * 商家端分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    Pagination<OmsOrderDTO> getMerchantOrderPaging(OrderPagingParam param);

    /**
     * 商家订单核验
     * @param orderId
     * @param merchantId
     */
    void orderVerification(Long orderId, Long merchantId);

    /**
     * 查询商家端订单管理
     * @param merchantId
     * @return
     */
    OrderDTO queryOrderManage(Long merchantId);

    /**
     * 商家端我的团队
     * @param merchantId
     * @return
     */
    MerchantTeamDTO queryTeamOrder(Long merchantId);


    /**
     * 根据核销码获取订单详情
     */
    OmsOrderDetailDTO getOraderDetail(String orderSn,Long merchantId);

    /**
     * 下单回调接口
     * @param orderResponse
     */
    void orderCallBacl(OrderResponse orderResponse);

    void orderNotify(SxyNotifyParam param);

    void orderRefund(@RequestBody OrderRefundParam param);

    void orderRefundNotify(SxyNotifyParam param);

    OmsOrder queryCheckOrder(Long orderId, Long id);
}
