package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.client.dto.WechatUnifiedOrder;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderItem;

import java.util.List;

/**
 * 首信易支付
 *
 * @author liude
 * @date 2025/1/8
 */
public interface OnlinePayService{
    /**
     * 商户发送请求参数给首信易支付，并接收同步返回的支付信息，用支付信息拉起对应支付方式，并在订单终态后接收异步通知结果。
     */
    WechatUnifiedOrder order(OmsOrder omsOrder, List<OmsOrderItem> orderItemList, OmsOrderModifyParam orderParam);

    /**
     * 商户通过单笔订单查询接口查询首信易支付平台的订单状态，并且通过主动查询订单来获知订单信息
     * @param omsOrder
     */
    String query(OmsOrder omsOrder);

    /**
     * 提交退款申请接口用于商户端通过应用接口程序，实现提交退款申请的业务处理，
     * 首信易支付平台通过JSON格式方式返回退款申请提交的结果响应。其操作功能与现有首信易支付商户后台输入退款申请功能相同。
     */
    void refund(OmsOrder omsOrder, Long refundId);

    /**
     * 商户通过退款订单查询接口查询首信易支付平台的订单退款交易明细。
     * 商户通过主动查询订单获知订单退款信息
     * @param omsOrder
     * @return
     */
    String refundQuery(OmsOrder omsOrder);

    /**
     * 商户可调用此接口将未付款成功订单的置为取消
     * @param omsOrder
     * @return
     */
    String cancelOrder(OmsOrder omsOrder);
}
