package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.QueryMerchantBalanceParam;
import com.chelvc.cloud.vehicle.server.dao.BalanceDetailMapper;
import com.chelvc.cloud.vehicle.server.entity.Asset;
import com.chelvc.cloud.vehicle.server.entity.BalanceDetail;
import com.chelvc.cloud.vehicle.server.service.AssetService;
import com.chelvc.cloud.vehicle.server.service.BalanceDetailService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.DateUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class BalanceDetailServiceImpl extends ServiceImpl<BalanceDetailMapper, BalanceDetail>
        implements BalanceDetailService{
    private final BalanceDetailMapper merchantBalanceDetailMapper;
    private final MerchantService merchantService;
    private final AssetService assetService;
    @Override
    public void recordFlow(Long orderId, Integer type, Long targetId, BigDecimal operateAmount, BigDecimal deduct,
                           BigDecimal realityAmount, BigDecimal surplusAmount,
                           Integer flowType, Integer operateType, Long sourceId) {
        BalanceDetail detail = new BalanceDetail();
        detail.setType(type);
        detail.setFlowType(flowType);
        detail.setOperateType(operateType);
        detail.setOrderId(orderId);
        detail.setTargetId(targetId);
        detail.setCreateTime(new Date());
        detail.setOperateAmount(operateAmount);
        detail.setDeduct(deduct);
        detail.setRealityAmount(realityAmount);
        detail.setSurplusAmount(surplusAmount);
        detail.setSourceId(sourceId);
        baseMapper.insert(detail);
    }

    @Override
    public Pagination<BalanceDetailDTO> queryByMerchantId(QueryMerchantBalanceParam param,
                    Integer pageNum, Integer pageSize, Integer type) {
        Long merchantId = param.getMerchantId();
        if(merchantId == null){
            return null;
        }
        Page<BalanceDetailDTO> page = new Page<>(pageNum, pageSize);
        IPage<BalanceDetailDTO> result = merchantBalanceDetailMapper
                .queryByUserId(page, queryWrapper(param, merchantId, type));
        return Pagination.<BalanceDetailDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    @Override
    public WalletDTO getMerchantAsset(Long merchantId) {
//        LocalDate today = LocalDate.now(); // 获取今天的日期
//        LocalDateTime midnight = LocalDateTime.of(today, LocalTime.MIDNIGHT); // 获取今天凌晨的时间
//        LocalDate yesterday = today.minusDays(1);
//        LocalDateTime yesterdayMidnight = LocalDateTime.of(yesterday, LocalTime.MIN);
        return merchantBalanceDetailMapper.getMerchantAsset(merchantId);
    }

    @Override
    public List<BalanceRecordDTO> listBalanceRecord(Integer type, Long date) {
        Date localDate = new Date(date);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Long userId = SessionContextHolder.getId();
        if (2 == type){
            List<Long> merchantIds = this.merchantService.getMerchantIdsByUserId(userId);
            if (CollectionUtils.isEmpty(merchantIds)){
                return new ArrayList<>();
            }
            List<BalanceDetail> balanceDetails = this.lambdaQuery()
                    .eq(BalanceDetail::getTargetId,merchantIds.get(0))
                    .eq(BalanceDetail::getType, type)
                    .orderByDesc(BalanceDetail::getCreateTime)
                    .list();
            List<BalanceRecordDTO> list = convert(balanceDetails);
            return list.stream()
                    .filter(item -> sdf.format(item.getCreateTime()).equals(sdf.format(localDate)))
                    .collect(Collectors.toList());
        }else{
            List<BalanceDetail> balanceDetails = this.lambdaQuery()
                    .eq(BalanceDetail::getTargetId,userId)
                    .eq(BalanceDetail::getType, type)
                    .orderByDesc(BalanceDetail::getCreateTime)
                    .list();
            List<BalanceRecordDTO> list = convert(balanceDetails);
            return list.stream()
                    .filter(item -> !DateUtils.format(item.getCreateTime()).equals(DateUtils.format(localDate)))
                    .collect(Collectors.toList());
        }
    }

    @Override
    public EarningsDTO getBalanceRecord(Integer type) {
        Long userId = SessionContextHolder.getId();
        EarningsDTO balanceRecord = this.baseMapper.getBalanceRecord(userId, type);
        if(balanceRecord == null){
            balanceRecord = new EarningsDTO();
        }
        Asset asset = assetService.queryAsset(userId, type);
        if(asset == null){
            balanceRecord.setWithdrawAmount(BigDecimal.ZERO);
        } else {
            balanceRecord.setWithdrawAmount(asset.getTotal());
        }
        return balanceRecord;
    }

    @Override
    public BalanceDetail queryBy(Long orderId, Integer type, Long userId, Integer flowType, Integer operateType) {
        LambdaQueryWrapper<BalanceDetail> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(BalanceDetail::getOrderId, orderId);
        wrapper.eq(BalanceDetail::getTargetId, userId);
        wrapper.eq(BalanceDetail::getFlowType, flowType);
        wrapper.eq(BalanceDetail::getType, type);
        wrapper.eq(BalanceDetail::getOperateType, operateType);
        return baseMapper.selectOne(wrapper);
    }

    public QueryWrapper<BalanceDetail> queryWrapper(QueryMerchantBalanceParam param, Long merchantId, Integer type) {
        QueryWrapper<BalanceDetail> lqw = Wrappers.query();
        lqw.eq("t1.type", type);
        lqw.eq("t1.target_id", merchantId);
        if(param.getFlowType() != null){
            lqw.eq("t1.flow_type", param.getFlowType());
        }
        if(param.getOperateType() != null){
            lqw.eq("t1.operate_type", param.getOperateType());
        }
        if(param.getStartTime() != null){
            lqw.ge("t1.create_time", param.getStartTime());
        }
        if(param.getEndTime() != null){
            lqw.lt("t1.create_time", param.getEndTime());
        }
        lqw.orderByDesc("t1.create_time");
        return lqw;
    }

    public List<BalanceRecordDTO> convert(List<BalanceDetail> balanceDetails){
        List<BalanceRecordDTO> list = new ArrayList<BalanceRecordDTO>( balanceDetails.size() );
        for (BalanceDetail balanceDetail : balanceDetails) {
            BalanceRecordDTO dto = new BalanceRecordDTO();
            dto.setId(balanceDetail.getId());
            dto.setDeduct(balanceDetail.getDeduct());
            dto.setAmount(balanceDetail.getOperateAmount());
            dto.setCurrencyType(balanceDetail.getFlowType());
            dto.setRemark(balanceDetail.getRemark());
            dto.setRealityAmount(balanceDetail.getRealityAmount());
            dto.setSourceType(balanceDetail.getOperateType());
            dto.setCreateTime(balanceDetail.getCreateTime());
            dto.setUserId(balanceDetail.getTargetId());
            dto.setOrderId(balanceDetail.getOrderId());
            dto.setType(balanceDetail.getType());
            dto.setSourceUserId(balanceDetail.getSourceId());
            list.add(dto);
        }
        return list;
    }
}
