package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.MerchantApplyStatus;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.server.dao.MerchantApplyRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantApplyRecord;
import com.chelvc.cloud.vehicle.server.entity.MerchantAuth;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.upay.sdk.Constants;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantApplyRecordServiceImpl extends
        ServiceImpl<MerchantApplyRecordMapper, MerchantApplyRecord> implements MerchantApplyRecordService {

    private final MerchantRelationService merchantRelationService;
    private final UserAuthService userAuthService;
    private final MerchantAuthService merchantAuthService;
    private final AssetService assetService;

    @Override
    public int insert(MerchantApplyRecord record) {
        return baseMapper.insert(record);
    }

    @Override
    public String updateSxyMerchantIdByNotify(SxyNotifyParam param) {
        JSONObject responseData = TradeHandle.decrypt(param);
        log.info("申请商户号回调参数：{}", responseData);
        String status = responseData.getString("subMerchantReviewStatus");
        if (Constants.ERROR.equals(status)) {
            //解密后报文，如果status=ERROR时，不需要验签
            return status;
        }
        String requestId = responseData.getString("requestId");
        LambdaQueryWrapper<MerchantApplyRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantApplyRecord::getRequestId, requestId);
        MerchantApplyRecord record = baseMapper.selectOne(wrapper);
        Integer recordStatus = record.getStatus();
        MerchantAuth merchantAuth = merchantAuthService.queryByUserId(record.getUserId());
        if(merchantAuth == null){
            log.error("商户号申请记录id【{}】查询商户认证信息为空", requestId);
            return status;
        }
        if(MerchantApplyStatus.SUCCESS.getStatus().equals(recordStatus)
            && String.valueOf(MerchantApplyStatus.SUCCESS.getStatus()).equals(merchantAuth.getReviewStatus())){
            return MerchantApplyStatus.SUCCESS.getType();
        }
        String subMerchantId = responseData.getString("subMerchantId");
        Integer type = record.getType();
        Integer resultStatus = MerchantApplyStatus.getStatus(status);
        record.setStatus(resultStatus);
        record.setExamineStatus(status);
        record.setCertificateSupplementUrl(responseData.getString("certificateSupplementUrl"));
        record.setElectronicContractingUrl(responseData.getString("electronicContractingUrl"));
        String merchantReviewRemarks = responseData.getString("merchantReviewRemarks");
        record.setMerchantReviewRemarks(merchantReviewRemarks);
        record.setPostReviewRemark(responseData.getString("postReviewRemark"));
        record.setPostReviewStatus(responseData.getString("postReviewStatus"));
        record.setSubMerchantId(subMerchantId);
        record.setUpdateTime(new Date());
        DatabaseContextHolder.transactional((Executor) () -> {
            baseMapper.updateById(record);
            if(type == 1){
                //更改个人认证状态
                int i = userAuthService.updateStatus(record.getUserId(), resultStatus, merchantReviewRemarks);
                if(i != 1){
                    log.error("用户id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("用户id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
            }
            if(type == 2){
                //更改商家认证状态
                int i = merchantAuthService.updateStatus(record.getUserId(), resultStatus, merchantReviewRemarks);
                if(i != 1){
                    log.error("商家id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("商家id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
            }
            if("SUCCESS".equals(status)){
                int i = merchantRelationService.insert(record.getUserId(), record.getType(), subMerchantId);
                if(i != 1){
                    log.error("商家id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("商家id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
                if(record.getType() == 1){
                    //用户申请商户号成功，平台转账给用户
                    BigDecimal total = assetService.queryTotal(record.getUserId(), 1);
                    assetService.transfer(0L, record.getUserId(), 1, total, "0");
                }
            }
        });
        //判断状态是否是通过 通过调用电子签重签
        if("PASS".equals(status)){
            merchantRelationService.reSign(requestId, subMerchantId, 5);
        }
        return status;
    }

    @Override
    public String updateStatusByReSignNotify(Integer frequency, SxyNotifyParam param) {
        if(frequency == null || frequency > 5 || frequency < 1){
            return "success";
        }
        JSONObject responseData = TradeHandle.decrypt(param);
        log.info("电子签重签回调参数：{}", responseData);
        String status = responseData.getString("status");
        String subMerchantId = responseData.getString("subMerchantId");
        String requestId = responseData.getString("requestId");
        if (!Constants.SUCCESS.equals(status)) {
            merchantRelationService.reSign(requestId, subMerchantId, --frequency);
            return status;
        }
        LambdaQueryWrapper<MerchantApplyRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantApplyRecord::getRequestId, requestId);
        MerchantApplyRecord record = baseMapper.selectOne(wrapper);
        if(record == null){
            return "FAIL";
        }
        Integer type = record.getType();
        Integer resultStatus = MerchantApplyStatus.getStatus(status);
        record.setStatus(resultStatus);
        record.setExamineStatus(status);
        String errorMessage = responseData.getString("errorMessage");
        DatabaseContextHolder.transactional((Executor) () -> {
            baseMapper.updateById(record);
            if(type == 1){
                //更改个人认证状态
                int i = userAuthService.updateStatus(record.getUserId(), resultStatus, errorMessage);
                if(i != 1){
                    log.error("用户id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("用户id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
            }
            if(type == 2){
                //更改商家认证状态
                int i = merchantAuthService.updateStatus(record.getUserId(), resultStatus, errorMessage);
                if(i != 1){
                    log.error("商家id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("商家id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
            }
            if("SUCCESS".equals(status)){
                int i = merchantRelationService.insert(record.getUserId(), record.getType(), subMerchantId);
                if(i != 1){
                    log.error("商家id【{}】更新认证状态【{}】失败", record.getUserId(), status);
                    throw new ResourceUnavailableException("商家id【{"+record.getUserId()+"}】更新认证状态【{"+status+"}】失败");
                }
                if(record.getType() == 1){
                    //用户申请商户号成功，平台转账给用户
                    BigDecimal total = assetService.queryTotal(record.getUserId(), 1);
                    assetService.transfer(0L, record.getUserId(), 1, total, "0");
                }
            }
        });
        return status;
    }
}
