package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.TradeConstant;
import com.chelvc.cloud.vehicle.client.util.SFTPUtils;
import com.chelvc.cloud.vehicle.server.dao.MerchantAuthMapper;
import com.chelvc.cloud.vehicle.server.dao.MerchantRelationMapper;
import com.chelvc.cloud.vehicle.server.dao.UserAuthMapper;
import com.chelvc.cloud.vehicle.server.entity.*;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.MerchantApplyRecordService;
import com.chelvc.cloud.vehicle.server.service.MerchantRelationService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

@Service
@Slf4j
@RequiredArgsConstructor(onConstructor = @__({@Autowired,@Lazy}))
public class MerchantRelationServiceImpl extends ServiceImpl<MerchantRelationMapper,
        MerchantRelation> implements MerchantRelationService {
    @Value("${chelvc.domain}")
    private String NOTIFY_DOMAIN;
    private final MerchantService merchantService;
    private final MerchantApplyRecordService merchantApplyRecordService;
    private final MerchantAuthMapper merchantAuthMapper;
    private final UserAuthMapper userAuthMapper;
    @Override
    public MerchantRelation queryByUserIdAndType(Long userId, Integer type) {
        LambdaQueryWrapper<MerchantRelation> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantRelation::getUserId, userId);
        wrapper.eq(MerchantRelation::getType, type);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public void registerMerchantId(MerchantAuth merchantAuth) {
        Merchant merchant = merchantService.queryByApplicant(merchantAuth.getApplicant());
        String requestId = UUID.randomUUID().toString().replaceAll("-", "");
        JSONObject jsonObject = new JSONObject();
        String operationType = "CREATE";
        jsonObject.put("merchantId", TradeConstant.PARTNER_ID);
        jsonObject.put("requestId", requestId);
        jsonObject.put("operationType", operationType);
        jsonObject.put("notifyUrl", NOTIFY_DOMAIN + TradeConstant.MERCHANT_NET_IN_NOTIFY_URL);
        jsonObject.put("extendedParameters", "sendActiveEmail:FALSE,sendExpressPayMsg:FALSE");

        //商户基本信息
        JSONObject baseInfo = new JSONObject();
        baseInfo.put("signedType", "BY_SPLIT_BILL");
        baseInfo.put("signedName", merchantAuth.getBusinessName());
        baseInfo.put("registerRole", "INDIVIDUAL_BUSINESS"); //个体工商户
        baseInfo.put("cerType", "INDIVIDUAL_BUSINESS_LICENCE"); //个体工商户营业执照
        baseInfo.put("signedShorthand", merchantAuth.getStoreName());
        baseInfo.put("businessAddressProvince", merchant.getProvince());
        baseInfo.put("businessAddressCity", merchant.getCity());
        baseInfo.put("businessAddressArea", merchant.getDistrict());
        baseInfo.put("businessAddress", merchant.getAddress());
        baseInfo.put("contactName", merchant.getLiaison());
        baseInfo.put("contactEmail", StringUtils.isNotBlank(merchantAuth.getEmail()) ? merchantAuth.getEmail() : merchant.getMobile() + "@qq.com");
        baseInfo.put("contactPhone", merchant.getMobile());
        jsonObject.put("baseInfo", baseInfo);

        //结算信息
        JSONObject bankCardInfo = new JSONObject();
        bankCardInfo.put("accountName", merchantAuth.getLegalRepresentativeName()); //开户名称
        bankCardInfo.put("bankCardNo", merchantAuth.getBankCardNo()); //开户账号
        bankCardInfo.put("accountType", "PRIVATE");
        bankCardInfo.put("liquidationType", "WITHDRAW");
        bankCardInfo.put("withdrawRateType", "SINGLE");
        bankCardInfo.put("withdrawRate", "1.00");
        bankCardInfo.put("reservedPhoneNo", merchantAuth.getReservedPhoneNo()); //银行预留手机号
        jsonObject.put("bankCardInfo", bankCardInfo);

        //商户资质信息
        JSONObject certificateInfo = new JSONObject();
        certificateInfo.put("legalPersonName", merchantAuth.getLegalRepresentativeName());
        certificateInfo.put("legalPersonPhone", merchantAuth.getReservedPhoneNo());
        certificateInfo.put("legalPersonIdType", "IDCARD");
        certificateInfo.put("legalPersonIdNo", merchantAuth.getIdCardNumber());
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date idCardExpirationDateStart = merchantAuth.getIdCardExpirationDateStart();
        certificateInfo.put("idEffectiveDateStart", formatter.format(idCardExpirationDateStart));  //身份证号有效期开始
        Date idCardExpirationDate = merchantAuth.getIdCardExpirationDate();
        certificateInfo.put("idEffectiveDateEnd", formatter.format(idCardExpirationDate));
        certificateInfo.put("cerNo", merchantAuth.getBusinessLicenseId());
        certificateInfo.put("sellingArea", "C");
        certificateInfo.put("staffSize", "C");
        String businessPhoto = merchantAuth.getBusinessPhoto();
        String businessPhotoPath = SFTPUtils.ftpUpload(businessPhoto,
                merchant.getId() + "-businessLicensePath." + SFTPUtils.getSuffix(businessPhoto));
        certificateInfo.put("businessLicensePath", businessPhotoPath); //营业执照照片路径

        /*String businessLicensePhoto = merchantAuth.getBusinessLicensePhoto();
        String openAccountPath = SFTPUtils.ftpUpload(businessLicensePhoto,
                merchant.getId() + "-openAccountPath." + SFTPUtils.getSuffix(businessLicensePhoto));
        certificateInfo.put("openAccountPath", openAccountPath);*/  //	开户许可证照片路径

        String idCardFrontPhoto = merchantAuth.getIdCardFrontPhoto();
        String legalIdCardProsPath = SFTPUtils.ftpUpload(idCardFrontPhoto,
                merchant.getId() + "-legalIdCardProsPath." + SFTPUtils.getSuffix(idCardFrontPhoto));
        certificateInfo.put("legalIdCardProsPath", legalIdCardProsPath);  //	法人证件人像面路径

        String idCardBackPhoto = merchantAuth.getIdCardBackPhoto();
        String legalIdCardConsPath = SFTPUtils.ftpUpload(idCardBackPhoto,
                merchant.getId() + "-legalIdCardConsPath." + SFTPUtils.getSuffix(idCardBackPhoto));
        certificateInfo.put("legalIdCardConsPath", legalIdCardConsPath);  //	法人证件国徽面路径
        jsonObject.put("certificateInfo", certificateInfo);

        //商户合同信息
        JSONObject contractInfo = new JSONObject();
        contractInfo.put("receiverName", merchant.getLiaison());
        contractInfo.put("receiverPhone", merchant.getMobile());
        contractInfo.put("receiverAddress", merchant.getAddress());
        jsonObject.put("contractInfo", contractInfo);

        JSONObject json = TradeHandle.handle(String.valueOf(jsonObject), TradeConstant.DECLARE_URL,
                TradeConstant.PARTNER_ID, null);
        String status = json.getString("status");
        int resultStatus = 0;
        String examineStatus = "PROCESSING";
        String errorMessage = "";
        if(!"SUCCESS".equals(status)) {
            resultStatus = 4;
            examineStatus = "NO_PASS";
            errorMessage = json.getString("errorMessage");
            if("".equals(errorMessage)){
                errorMessage = json.getString("cause");
            }
            log.info("店名：【{}】-【认证失败】:{}", merchantAuth.getStoreName(), errorMessage);
            //更改状态
            merchantAuthMapper.updateStatusById(merchantAuth.getId(), String.valueOf(resultStatus), errorMessage);
        }
        String subMerchantId = json.getString("subMerchantId");
        MerchantApplyRecord record = new MerchantApplyRecord();
        record.setUserId(merchantAuth.getApplicant());
        record.setType(2);
        record.setRequestId(requestId);
        record.setStatus(resultStatus);
        record.setOperationType("CREATE");
        record.setExamineStatus(examineStatus);
        record.setSubMerchantId(subMerchantId);
        record.setMerchantReviewRemarks(errorMessage);
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        merchantApplyRecordService.insert(record);
    }

    @Override
    public void registerMerchantId(UserAuth userAuth) {
        String requestId = UUID.randomUUID().toString().replaceAll("-", "");
        JSONObject jsonObject = new JSONObject();
        String operationType = "CREATE";
        jsonObject.put("merchantId", TradeConstant.PARTNER_ID);
        jsonObject.put("requestId", requestId);
        jsonObject.put("operationType", operationType);
        jsonObject.put("notifyUrl", NOTIFY_DOMAIN + TradeConstant.MERCHANT_NET_IN_NOTIFY_URL);
        jsonObject.put("extendedParameters", "sendActiveEmail:FALSE,sendExpressPayMsg:FALSE");

        //商户基本信息
        JSONObject baseInfo = new JSONObject();
        baseInfo.put("signedType", "BY_SPLIT_BILL");
        baseInfo.put("signedName", userAuth.getUserName());
        baseInfo.put("registerRole", "NATURAL_PERSON"); //个体工商户
        baseInfo.put("signedShorthand", userAuth.getUserName()); //商户简称
        baseInfo.put("businessAddressProvince", userAuth.getProvince()); //省
        baseInfo.put("businessAddressCity", userAuth.getCity());    //市
        baseInfo.put("businessAddressArea", userAuth.getDistrict()); //区
        baseInfo.put("businessAddress", userAuth.getAddress());    //经营地址
        baseInfo.put("contactName", userAuth.getUserName()); //联系人姓名
        baseInfo.put("contactEmail", StringUtils.isNotBlank(userAuth.getEmail()) ? userAuth.getEmail() : userAuth.getPhone() + "@qq.com");  //联系人邮箱
        baseInfo.put("contactPhone", userAuth.getPhone()); //联系人电话
        jsonObject.put("baseInfo", baseInfo);

        //结算信息
        JSONObject bankCardInfo = new JSONObject();
        bankCardInfo.put("accountName", userAuth.getUserName()); //开户名称
        bankCardInfo.put("bankCardNo", userAuth.getBankCardNo()); //开户账号
        bankCardInfo.put("liquidationType", "WITHDRAW");
        bankCardInfo.put("withdrawRateType", "SINGLE");
        bankCardInfo.put("withdrawRate", "1.00");
        jsonObject.put("bankCardInfo", bankCardInfo);

        //商户资质信息
        JSONObject certificateInfo = new JSONObject();
        certificateInfo.put("legalPersonName", userAuth.getUserName()); //法人姓名
        certificateInfo.put("profession", "2");  //职业 固定销售
        certificateInfo.put("legalPersonIdType", "IDCARD");
        certificateInfo.put("legalPersonIdNo", userAuth.getIdCardNo());  //身份证号
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date idCardExpirationDateStart = userAuth.getIdCardExpirationDateStart();
        certificateInfo.put("idEffectiveDateStart", formatter.format(idCardExpirationDateStart));  //身份证号有效期开始
        Date idCardExpirationDate = userAuth.getIdCardExpirationDate();
        certificateInfo.put("idEffectiveDateEnd", formatter.format(idCardExpirationDate));

        String idCardFrontPhoto = userAuth.getIdCardFrontPhoto();
        String legalIdCardProsPath = SFTPUtils.ftpUpload(idCardFrontPhoto,
                userAuth.getUserId() + "-legalIdCardProsPath." + SFTPUtils.getSuffix(idCardFrontPhoto));
        certificateInfo.put("legalIdCardProsPath", legalIdCardProsPath);  //	法人证件人像面路径

        String idCardBackPhoto = userAuth.getIdCardBackPhoto();
        String legalIdCardConsPath = SFTPUtils.ftpUpload(idCardBackPhoto,
                userAuth.getUserId() + "-legalIdCardConsPath." + SFTPUtils.getSuffix(idCardBackPhoto));
        certificateInfo.put("legalIdCardConsPath", legalIdCardConsPath);  //	法人证件国徽面路径
        jsonObject.put("certificateInfo", certificateInfo);

        //商户合同信息
        JSONObject contractInfo = new JSONObject();
        contractInfo.put("receiverName", userAuth.getUserName());
        contractInfo.put("receiverPhone", userAuth.getPhone());
        contractInfo.put("receiverAddress", userAuth.getAddress());
        jsonObject.put("contractInfo", contractInfo);

        JSONObject json = TradeHandle.handle(String.valueOf(jsonObject), TradeConstant.DECLARE_URL,
                TradeConstant.PARTNER_ID, null);
        String status = json.getString("status");
        int resultStatus = 0;
        String examineStatus = "PROCESSING";
        String errorMessage = "";
        if(!"SUCCESS".equals(status)){
            resultStatus = 4;
            examineStatus = "NO_PASS";
            errorMessage = json.getString("errorMessage");
            if("".equals(errorMessage)){
                errorMessage = json.getString("cause");
            }
            log.info("用户：【{}】-【认证失败】:{}", userAuth.getUserName(), errorMessage);
            //更改状态
            userAuthMapper.updateStatusById(userAuth.getId(), resultStatus, errorMessage);
        }
        String subMerchantId = json.getString("subMerchantId");
        MerchantApplyRecord record = new MerchantApplyRecord();
        record.setUserId(userAuth.getUserId());
        record.setType(1);
        record.setRequestId(requestId);
        record.setStatus(resultStatus);
        record.setOperationType("CREATE");
        record.setExamineStatus(examineStatus);
        record.setSubMerchantId(subMerchantId);
        record.setMerchantReviewRemarks(errorMessage);
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        merchantApplyRecordService.insert(record);
    }

    @Override
    public int insert(Long userId, Integer type, String sxyMerchantId) {
        LambdaQueryWrapper<MerchantRelation> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantRelation::getType, type);
        wrapper.eq(MerchantRelation::getUserId, userId);
        MerchantRelation merchantRelation = baseMapper.selectOne(wrapper);
        if(merchantRelation != null){
            merchantRelation.setSxyMerchantId(sxyMerchantId);
            return baseMapper.updateById(merchantRelation);
        }
        MerchantRelation relation = new MerchantRelation();
        relation.setType(type);
        relation.setUserId(userId);
        relation.setSxyMerchantId(sxyMerchantId);
        return baseMapper.insert(relation);
    }

    @Override
    public void reSign(String requestId, String subMerchantId, int frequency) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("requestId", requestId);
        jsonObject.put("subMerchantId", subMerchantId);
        jsonObject.put("eSignNotifyUrl", NOTIFY_DOMAIN + TradeConstant.MERCHANT_RE_SIGN_NOTIFY_URL + frequency);
        jsonObject.put("merchantId", TradeConstant.PARTNER_ID);
        JSONObject json = TradeHandle.handle(String.valueOf(jsonObject), TradeConstant.RE_SIGN_URL,
                TradeConstant.PARTNER_ID, null);
        log.info("子商户【{}】电子签重签返回信息：{}", subMerchantId, json);
    }
}
