package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.param.OrderRefundParam;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderRefundMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderRefund;
import com.chelvc.cloud.vehicle.server.service.OmsOrderRefundService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderRefundServiceImpl extends ServiceImpl<OmsOrderRefundMapper, OmsOrderRefund> implements OmsOrderRefundService {

    @Override
    public Long insert(OmsOrder omsOrder, OrderRefundParam param) {
        OmsOrderRefund refund = new OmsOrderRefund();
        refund.setOrderId(omsOrder.getId());
        refund.setRefundAmount(omsOrder.getPayAmount());
        refund.setMerchantId(omsOrder.getMerchantId());
        refund.setUserId(omsOrder.getUserId());
        refund.setReason(param.getReason());
        refund.setProofPics(param.getProofPics());
        refund.setStatus(0);
        refund.setCreateTime(new Date());
        refund.setHandleTime(new Date());
        int i = baseMapper.insert(refund);
        if(i != 1){
            throw new ResourceUnavailableException("退款失败，请稍后重试");
        }
        return refund.getId();
    }

    @Override
    public int updateStatus(Long refundId, Integer status, String cause) {
        UpdateWrapper<OmsOrderRefund> update = Wrappers.update();
        update.set("status", status);
        update.set("handle_note", cause);
        update.eq("id", refundId);
        return baseMapper.update(null, update);
    }

    @Override
    public int updateStatus(Long refundId, Integer status, String cause, String serialNumber) {
        UpdateWrapper<OmsOrderRefund> update = Wrappers.update();
        update.set("status", status);
        update.set("handle_note", cause);
        update.set("serial_number", serialNumber);
        update.eq("id", refundId);
        return baseMapper.update(null, update);
    }

    @Override
    public OmsOrderRefund queryById(Long id) {
        return baseMapper.selectById(id);
    }

    @Override
    public int queryApplyOrder(Long orderId) {
        return baseMapper.queryApplyOrder(orderId);
    }
}
