package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.user.client.model.Scope;
import com.chelvc.cloud.vehicle.client.constant.CouponType;
import com.chelvc.cloud.vehicle.client.constant.ReserveStatus;
import com.chelvc.cloud.vehicle.client.constant.UserCouponStatus;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderPagingParam;
import com.chelvc.cloud.vehicle.client.param.OrderRefundParam;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.client.util.DateTimeUtils;
import com.chelvc.cloud.vehicle.server.copier.MerchantCopier;
import com.chelvc.cloud.vehicle.server.copier.OrderCopier;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderMapper;
import com.chelvc.cloud.vehicle.server.entity.*;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.upay.sdk.onlinepay.builder.OrderResponse;
import io.netty.util.HashedWheelTimer;
import io.netty.util.Timeout;
import io.netty.util.Timer;
import io.netty.util.TimerTask;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.Asserts;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 订单Service业务层处理
 *
 * @author liude
 * @date 2023-11-08
 */
@Service
@Slf4j
@RequiredArgsConstructor(onConstructor = @__({@Lazy, @Autowired}))
public class OmsOrderServiceImpl extends ServiceImpl<OmsOrderMapper, OmsOrder> implements OmsOrderService{
    private final OmsOrderMapper omsOrderMapper;
    private final OmsCartItemService omsCartItemService;
    private final OmsOrderSettingService orderSettingService;
    private final OmsOrderItemService omsOrderItemService;
    private final UserCouponService userCouponService;
    private final MerchantService merchantService;
    private final OrderHandleService orderHandleService;
    private final OmsOrderOperateHistoryService omsOrderOperateHistoryService;
    private final AssessService assessService;
    private final AssetService assetService;
    private final UserClient userService;
    private final UserReceiveAddressService userReceiveAddressService;
    private final ReservationService reservationService;
    private final BalanceDetailService balanceDetailService;
    private final OmsOrderRefundService orderRefundService;

    private final GoodsService goodsService;

    // 定义字符池
    private static final String LOWER_CASE = "abcdefghijklmnopqrstuvwxyz";
    private static final String UPPER_CASE = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    private static final String DIGITS = "0123456789";
    private static final String ALL_CHARACTERS = LOWER_CASE + UPPER_CASE + DIGITS;

    private final OnlinePayService onlinePayService;

    @Override
    public ConfirmOrderResultDTO generateConfirmOrder(OmsOrderModifyParam orderParam) {
        ConfirmOrderResultDTO result = new ConfirmOrderResultDTO();
        //获取购物车信息
        Long userId = SessionContextHolder.getId();
        List<OmsCartItemDTO> cartItemList = omsCartItemService.listPromotion(orderParam.getCartIds());
        result.setCartItemList(cartItemList);
//        //获取用户收货地址列表
//        List<UmsMemberReceiveAddress> memberReceiveAddressList = memberReceiveAddressService.list();
//        result.setMemberReceiveAddressList(memberReceiveAddressList);
        //获取用户可用优惠券列表
//        List<SmsCouponHistoryDetail> couponHistoryDetailList = memberCouponService.listCart(cartPromotionItemList, 1);
//        result.setCouponHistoryDetailList(couponHistoryDetailList);
//        //获取用户积分
//        result.setMemberIntegration(currentMember.getIntegration());
//        //获取积分使用规则
//        UmsIntegrationConsumeSetting integrationConsumeSetting = integrationConsumeSettingMapper.selectByPrimaryKey
//        (PlatformConstant.);
//        result.setIntegrationConsumeSetting(integrationConsumeSetting);
//        //计算总金额、活动优惠、应付金额
        CalcAmountDTO calcAmount = calcCartAmount(cartItemList);
        result.setCalcAmount(calcAmount);
        List<UserCouponDTO> userCouponList = this.userCouponService.getUserCouponList(userId,
                calcAmount.getTotalAmount());
        result.setCouponHistoryDetailList(userCouponList);
        if (!StringUtils.isEmpty(orderParam.getMerchantId())) {
            Merchant merchant = this.merchantService.getById(orderParam.getMerchantId());
            result.setMerchantDTO(MerchantCopier.INSTANCE.copying(merchant));
        }
        return result;
    }

    @Override
    public WechatUnifiedOrder generateOrder(OmsOrderModifyParam orderParam) {
        log.info("订单参数：{}", orderParam);
        List<OmsOrderItem> orderItemList = new ArrayList<>();
        //获取购物车及优惠信息
        Long userId = SessionContextHolder.getId();
        synchronized (String.valueOf(userId).intern()){
            List<OmsCartItemDTO> cartPromotionItemList = new ArrayList<>();
            if (CollectionUtils.isEmpty(orderParam.getCartIds())){
                //生成下单商品信息
                OmsOrderItem orderItem = new OmsOrderItem();
                orderItem.setGoodsId(orderParam.getGoodsInfo().getGoodsId());
                orderItem.setGoodsName(orderParam.getGoodsInfo().getGoodsName());
                orderItem.setGoodsPic(orderParam.getGoodsInfo().getGoodsPic());
                orderItem.setGoodsPrice(orderParam.getGoodsInfo().getPrice());
                orderItem.setGoodsQuantity(orderParam.getGoodsInfo().getQuantity());
                orderItem.setGoodsCategoryId(orderParam.getGoodsInfo().getGoodsCategoryId());
//            orderItem.setPromotionAmount(cartPromotionItem.getReduceAmount());
//            orderItem.setPromotionName(cartPromotionItem.getPromotionMessage());
                orderItem.setGiftIntegration(0L);
                orderItem.setGiftGrowth(0);
                Long merchantId = goodsService.queryMerchantIdByGoodId(orderItem.getGoodsId());
                orderItem.setMerchantId(merchantId);
                orderItemList.add(orderItem);
            }else{
                cartPromotionItemList = omsCartItemService.listPromotion(orderParam.getCartIds());
                for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
                    //生成下单商品信息
                    OmsOrderItem orderItem = new OmsOrderItem();
                    orderItem.setGoodsId(cartPromotionItem.getGoodsId());
                    orderItem.setGoodsName(cartPromotionItem.getGoodsName());
                    orderItem.setGoodsPic(cartPromotionItem.getGoodsPic());
                    orderItem.setGoodsAttr(cartPromotionItem.getGoodsAttr());
                    orderItem.setGoodsBrand(cartPromotionItem.getGoodsBrand());
                    orderItem.setGoodsSn(cartPromotionItem.getGoodsSn());
                    orderItem.setGoodsPrice(cartPromotionItem.getPrice());
                    orderItem.setGoodsQuantity(cartPromotionItem.getQuantity());
                    orderItem.setGoodsSkuId(cartPromotionItem.getGoodsSkuId());
                    orderItem.setGoodsSkuCode(cartPromotionItem.getGoodsSkuCode());
                    orderItem.setGoodsCategoryId(cartPromotionItem.getGoodsCategoryId());
//            orderItem.setPromotionAmount(cartPromotionItem.getReduceAmount());
//            orderItem.setPromotionName(cartPromotionItem.getPromotionMessage());
                    orderItem.setGiftIntegration(0L);
                    orderItem.setGiftGrowth(0);
                    orderItem.setMerchantId(cartPromotionItem.getMerchantId());
                    orderItemList.add(orderItem);
                }
            }
            //判断购物车中商品是否都有库存
//        if (!hasStock(cartPromotionItemList)) {
//            Asserts.fail("库存不足，无法下单");
//        }
            //判断使用使用了优惠券
            boolean flag = false;
            if (orderParam.getUserCouponId() == null) {
                //不用优惠券
                for (OmsOrderItem orderItem : orderItemList) {
                    orderItem.setCouponAmount(new BigDecimal(0));
                }
            } else {
                UserCoupon coupon = userCouponService.checkUseCoupon(orderParam.getUserCouponId(), userId);
                if (coupon == null) {
                    Asserts.check(true, "该优惠券不可用");
                }
                //对下单商品的优惠券进行处理
                assert coupon != null;
                flag = handleCouponAmount(orderItemList, coupon);
            }
            //判断是否使用积分
            if (orderParam.getUseIntegration() == null || orderParam.getUseIntegration().equals(0)) {
                //不使用积分
                for (OmsOrderItem orderItem : orderItemList) {
                    orderItem.setIntegrationAmount(new BigDecimal(0));
                }
            } else {
                //使用积分
                BigDecimal totalAmount = calcTotalAmount(orderItemList);
//            BigDecimal integrationAmount = getUseIntegrationAmount(orderParam.getUseIntegration(), totalAmount,
//            currentMember, orderParam.getCouponId() != null);
//            if (integrationAmount.compareTo(new BigDecimal(0)) == 0) {
//                Asserts.check(true,"积分不可用");
//            } else {
                //可用情况下分摊到可用商品中
//                for (OmsOrderItemDTO orderItem : orderItemList) {
//                    BigDecimal perAmount = orderItem.getProductPrice().divide(totalAmount, 3, RoundingMode
//                    .HALF_EVEN).multiply(integrationAmount);
//                    orderItem.setIntegrationAmount(perAmount);
////                }
//            }
            }
            //计算order_item的实付金额
            handleRealAmount(orderItemList);
            //进行库存锁定
//        lockStock(cartPromotionItemList);
            //根据商品合计、运费、活动优惠、优惠券、积分计算应付金额
            OmsOrder order = new OmsOrder();
            order.setNote(orderParam.getNote());
            order.setDiscountAmount(new BigDecimal(0));
            order.setTotalAmount(calcTotalAmount(orderItemList));
            order.setFreightAmount(new BigDecimal(0));
            order.setPromotionAmount(calcPromotionAmount(orderItemList));
            order.setPromotionInfo(getOrderPromotionInfo(orderItemList));
            if (orderParam.getUserCouponId() == null || !flag) {
                order.setCouponAmount(new BigDecimal(0));
            } else {
                order.setCouponId(orderParam.getUserCouponId());
                order.setCouponAmount(calcCouponAmount(orderItemList));
            }
            if (orderParam.getUseIntegration() == null) {
                order.setIntegration(0);
                order.setIntegrationAmount(new BigDecimal(0));
            } else {
                order.setIntegration(orderParam.getUseIntegration());
                order.setIntegrationAmount(calcIntegrationAmount(orderItemList));
            }
            log.info("订单order:{}", order);
            order.setPayAmount(calcPayAmount(order));
            log.info("订单金额amount:{}", order.getPayAmount());
            //转化为订单信息并插入数据库
            order.setUserId(userId);
            order.setCreateTime(new Date());
            order.setMemberUsername(userId.toString());
            //支付方式：0->未支付；1->支付宝；2->微信；3->首信易（微信小程序）
            order.setPayType(orderParam.getPayType());
            //订单来源：0->PC订单；1->app订单
            order.setSourceType(1);
            //订单状态：0->待付款；1->待发货；2->已发货；3->已完成；4->已关闭；5->无效订单
            order.setStatus(0);
            //订单类型：0->正常订单；1->秒杀订单
            order.setOrderType(0);
            //收货人信息：姓名、电话、邮编、地址
            UserReceiveAddressDTO address = this.userReceiveAddressService.queryById(orderParam.getUserReceiveAddressId());
            if (null != address){
                order.setReceiverName(address.getReceiveName());
                order.setReceiverPhone(address.getPhone());
                order.setReceiverPostCode(address.getPostCode());
                order.setReceiverProvince(address.getProvince());
                order.setReceiverCity(address.getCity());
                order.setReceiverRegion(address.getRegion());
                order.setReceiverDetailAddress(address.getDetailAddress());
            }
            //0->未确认；1->已确认
            order.setConfirmStatus(0);
            order.setDeleteStatus(0);
            //计算赠送积分
            order.setIntegration(calcGifIntegration(orderItemList));
            //计算赠送成长值
            order.setGrowth(0);
            //生成订单号
            order.setOrderSn(generateOrderSn(6));
            order.setMerchantId(orderParam.getMerchantId());
            order.setId(RedisContextHolder.sequence());
            //设置自动收货天数
            List<OmsOrderSetting> orderSettings = orderSettingService.listOrderSetting();
            if (!CollectionUtils.isEmpty(orderSettings)) {
                order.setAutoConfirmDay(orderSettings.get(0).getConfirmOvertime());
            }
            //插入order表和order_item表
            if (!this.createIgnore(order)){
                order.setOrderSn(generateOrderSn(6));
                this.createIgnore(order);
            }
            for (OmsOrderItem orderItem : orderItemList) {
                orderItem.setOrderId(order.getId());
                orderItem.setOrderSn(order.getOrderSn());
            }
            omsOrderItemService.add(orderItemList);
            omsOrderOperateHistoryService.insertOmsOrderOperateHistory(order.getId(), userId, order.getStatus());
            //如使用优惠券更新优惠券使用状态
            if (orderParam.getUserCouponId() != null && flag) {
                updateCouponStatus(orderParam.getUserCouponId(), userId, 1);
            }
            //如使用积分需要扣除积分
//        if (orderParam.getUseIntegration() != null) {
//            order.setUseIntegration(orderParam.getUseIntegration());
//            memberService.updateIntegration(currentMember.getId(), currentMember.getIntegration() - orderParam
//            .getUseIntegration());
//        }
            //删除购物车中的下单商品
            deleteCartItemList(cartPromotionItemList, userId);
            //发送延迟消息取消订单
            sendDelayMessageCancelOrder(order.getId());
            //调用jsapi支付逻辑
//        WechatPayRequest request = new WechatPayRequest();
//        request.setAmount(order.getPayAmount());
//        request.setComment("商品购买");
//        request.setOrder(String.valueOf(order.getId()));
//        String openid = userService.queryOpenid(userId);
//        if(openid == null){
//            throw new ResourceUnavailableException("账号未绑定微信，请联系客服");
//        }
//        request.setOpenid(openid);
            //WechatPaymentHandler wechatPaymentHandler = applicationContext.getBean(WechatPaymentHandler.class);
            //return this.wechatPaymentHandler.unifiedorder("applet", request);
            return this.onlinePayService.order(order,orderItemList,orderParam);
        }
    }

    @Override
    public Integer paySuccess(Long orderId, Integer payType) {
        //修改订单支付状态
        OmsOrder order = this.baseMapper.selectById(orderId);
        if(order == null){
            return 0;
        }
        order.setStatus(1);
        order.setPaymentTime(new Date());
        order.setPayType(payType);
        DatabaseContextHolder.transactional((Executor) () ->
            orderHandleService.paySuccess(order)
        );
        //恢复所有下单商品的锁定库存，扣减真实库存
//        List<OmsOrderItemDTO> orderDetailList = this.baseMapper.getDetail(orderId);
//        int count = this.baseMapper.updateSkuStock(orderDetailList);
        return 1;
    }

    @Override
    public Integer cancelTimeOutOrder() {
        Integer count = 0;
        //查询订单设置
        List<OmsOrderSetting> orderSettings = orderSettingService.listOrderSetting();
        if (!CollectionUtils.isEmpty(orderSettings)) {
            Asserts.check(true, "未设置订单超时设置！");
        }
        //查询超时、未支付的订单及订单详情
        List<OmsOrderDetailDTO> timeOutOrders =
                this.baseMapper.getTimeOutOrders(orderSettings.get(0).getNormalOrderOvertime());
        if (CollectionUtils.isEmpty(timeOutOrders)) {
            return count;
        }
        //修改订单状态为交易取消
        List<Long> ids = new ArrayList<>();
        List<OmsOrderOperateHistory> histories = new ArrayList<>();
        for (OmsOrderDetailDTO timeOutOrder : timeOutOrders) {
            ids.add(timeOutOrder.getId());
            OmsOrderOperateHistory history = new OmsOrderOperateHistory();
            history.setOrderId(timeOutOrder.getId());
            history.setUserId(timeOutOrder.getUserId());
            history.setOrderStatus(5);
            history.setCreateTime(LocalDateTime.now());
            histories.add(history);
        }
        this.baseMapper.updateOrderStatus(ids, 6);
        omsOrderOperateHistoryService.batchInsertOmsOrderOperateHistory(histories);
        for (OmsOrderDetailDTO timeOutOrder : timeOutOrders) {
            //解除订单商品库存锁定
//            omsOrderMapper.releaseSkuStockLock(timeOutOrder.getOrderItemList());
            //修改优惠券使用状态
            updateCouponStatus(timeOutOrder.getCouponId(), timeOutOrder.getUserId(), 0);
            //返还使用积分
//            if (timeOutOrder.getUseIntegration() != null) {
//                UmsMember member = memberService.getById(timeOutOrder.getMemberId());
//                memberService.updateIntegration(timeOutOrder.getMemberId(), member.getIntegration() + timeOutOrder
//                .getUseIntegration());
//            }
        }
        return timeOutOrders.size();
    }

    @Override
    public void cancelOrder(Long orderId) {
        List<Long> orderIds = new ArrayList<>();
        orderIds.add(orderId);
        this.baseMapper.updateOrderStatus(orderIds, 0);
        // 初始化时间轮
        Timer timer = new HashedWheelTimer();
        // 定时任务
        TimerTask task1 = new io.netty.util.TimerTask() {
            public void run(Timeout timeout) throws Exception {
                // 取消订单业务逻辑
                closeOrder(orderId);
            }
        };
        // 注册此定时任务（延迟时间为900秒，也就是说900秒后订单过期）
        timer.newTimeout(task1, 900, TimeUnit.SECONDS);

    }

    private void closeOrder(Long orderId) {
        //查询未付款的取消订单
        List<OmsOrder> cancelOrderList = this.lambdaQuery()
                .eq(OmsOrder::getId,orderId)
                .eq(OmsOrder::getStatus,0)
                .list();
        if (CollectionUtils.isEmpty(cancelOrderList)) {
            return;
        }
        OmsOrder cancelOrder = cancelOrderList.get(0);
        if (cancelOrder != null) {
            //修改订单状态为交易取消
            List<Long> ids = new ArrayList<>();
            ids.add(cancelOrder.getId());
            this.baseMapper.updateOrderStatus(ids, 6);
            omsOrderOperateHistoryService.insertOmsOrderOperateHistory(cancelOrder.getId(), cancelOrder.getUserId(), 5);
            OmsOrderItem orderItem = new OmsOrderItem();
            orderItem.setOrderId(orderId);
//            List<OmsOrderItem> orderItemList = this.omsOrderItemService.selectByExample(orderItem);
            //解除订单商品库存锁定
//            if (!CollectionUtils.isEmpty(orderItemList)) {
//                this.baseMapper.releaseSkuStockLock(orderItemList);
//            }
            //修改优惠券使用状态
            updateCouponStatus(cancelOrder.getCouponId(), cancelOrder.getUserId(), 0);
            //返还使用积分
//            if (cancelOrder.getUseIntegration() != null) {
//                UmsMember member = memberService.getById(cancelOrder.getMemberId());
//                memberService.updateIntegration(cancelOrder.getMemberId(), member.getIntegration() + cancelOrder
//                .getUseIntegration());
//            }
        }
    }

    @Override
    public void sendDelayMessageCancelOrder(Long orderId) {
        //获取订单超时时间
//        OmsOrderSetting orderSetting = orderSettingService.selectByPrimaryKey(1L);
//        long delayTimes = orderSetting.getNormalOrderOvertime() * 60 * 1000;
        //发送延迟消息
//        cancelOrderSender.sendMessage(orderId, delayTimes);
    }

    @Override
    public void confirmReceiveOrder(Long orderId) {
        Long userId = SessionContextHolder.getId();
        OmsOrder order = this.baseMapper.selectOmsOrderById(orderId);
        if (!userId.equals(order.getUserId())) {
            Asserts.check(true, "不能确认他人订单！");
        }
        if (order.getStatus() != 3) {
            Asserts.check(true, "该订单还未发货！");
        }
        order.setStatus(4);
        order.setConfirmStatus(1);
        order.setReceiveTime(new Date());
        this.baseMapper.updateById(order);
        omsOrderOperateHistoryService.insertOmsOrderOperateHistory(orderId, userId, 4);
    }

    @Override
    public Pagination<OmsOrderDTO> getOrderPaging(OrderPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Integer status = param.getStatus();
        Page<OmsOrder> page = this.lambdaQuery()
                .eq(OmsOrder::getUserId, userId)
                .eq(StringUtils.notEmpty(status),OmsOrder::getStatus, status)
                .orderByDesc(OmsOrder::getCreateTime)
                .page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<OmsOrderDTO> pagination = DatabaseContextHolder.pagination(page, OrderCopier.INSTANCE::copying);
        pagination.getRecords().forEach(item -> {
            item.setMerchantInfo(this.merchantService.getMerchantById(item.getMerchantId()));
            List<OmsOrderItemDTO> goodsInfos = this.omsOrderItemService.getOrderItemByOrderId(item.getId());
            if(StringUtils.notEmpty(status) && status == 4){
                if(org.apache.commons.collections4.CollectionUtils.isNotEmpty(goodsInfos)){
                    goodsInfos.forEach(e ->
                            e.setAssessStatus(assessService.queryAssessStatus(e.getOrderId(), e.getGoodsId(), userId))
                    );
                }
            }
            item.setGoodsInfo(goodsInfos);
        });
        return pagination;
    }

    @Override
    public Pagination<OmsOrderDTO> getOmsOrderPaging(OrderPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Scope scope = StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
            Page<OmsOrder> page = this.lambdaQuery()
                    .eq(StringUtils.notEmpty(param.getStatus()), OmsOrder::getStatus, param.getStatus())
                    .like(StringUtils.notEmpty(param.getOrderSn()),OmsOrder::getOrderSn,param.getOrderSn())
                    .orderByDesc(OmsOrder::getCreateTime)
                    .page(DatabaseContextHolder.page(param.getPaging()));
            Pagination<OmsOrderDTO> pagination = DatabaseContextHolder.pagination(page, OrderCopier.INSTANCE::copying);
            pagination.getRecords().forEach(item -> {
                item.setGoodsInfo(this.omsOrderItemService.getOrderItemByOrderId(item.getId()));
            });
            return pagination;
        } else {
            List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            Page<OmsOrder> page = this.lambdaQuery()
                    .eq(StringUtils.notEmpty(param.getStatus()), OmsOrder::getStatus, param.getStatus())
                    .like(StringUtils.notEmpty(param.getOrderSn()),OmsOrder::getOrderSn,param.getOrderSn())
                    .in(OmsOrder::getMerchantId, merchants)
                    .orderByDesc(OmsOrder::getCreateTime)
                    .page(DatabaseContextHolder.page(param.getPaging()));
            Pagination<OmsOrderDTO> pagination = DatabaseContextHolder.pagination(page, OrderCopier.INSTANCE::copying);
            pagination.getRecords().forEach(item -> {
                item.setGoodsInfo(this.omsOrderItemService.getOrderItemByOrderId(item.getId()));
            });
            return pagination;
        }
    }

    @Override
    public OmsOrderDetailDTO detail(Long orderId) {
        OmsOrder omsOrder = this.baseMapper.selectById(orderId);
        List<OmsOrderItemDTO> orderItemList = omsOrderItemService.getOrderItemByOrderId(orderId);
        OmsOrderDetailDTO orderDetail = new OmsOrderDetailDTO();
        BeanUtils.copyProperties(omsOrder, orderDetail);
        orderDetail.setOrderItemList(orderItemList);
        MerchantDTO merchant = this.merchantService.getMerchantById(omsOrder.getMerchantId());
        orderDetail.setMerchantInfo(merchant);
        Integer count = this.baseMapper.countReservationByOrderId(orderId);
        if (count > 0){
            orderDetail.setReservation(true);
        }
        return orderDetail;
    }

    @Override
    public void deleteOrder(Long orderId) {
        Long userId = SessionContextHolder.getId();
        OmsOrder order = this.baseMapper.selectOmsOrderById(orderId);
        if (!userId.equals(order.getUserId())) {
            Asserts.check(true, "不能删除他人订单！");
        }
        if (order.getStatus() == 4 || order.getStatus() == 5) {
            order.setDeleteStatus(1);
            this.baseMapper.updateById(order);
        } else {
            Asserts.check(true, "只能删除已完成或已关闭的订单！");
        }
    }

    /**
     *
     */
    public static String generateOrderSn(int length) {
        if (length < 6) {
            throw new IllegalArgumentException("Password length must be at least 6 characters.");
        }

        SecureRandom random = new SecureRandom();
        StringBuilder password = new StringBuilder(length);

        // 确保每种类型至少有一个字符
        password.append(LOWER_CASE.charAt(random.nextInt(LOWER_CASE.length())));
        password.append(UPPER_CASE.charAt(random.nextInt(UPPER_CASE.length())));
        password.append(DIGITS.charAt(random.nextInt(DIGITS.length())));

        // 填充剩余的字符
        for (int i = 3; i < length; i++) {
            password.append(ALL_CHARACTERS.charAt(random.nextInt(ALL_CHARACTERS.length())));
        }

        // 将密码打乱顺序
        return shuffleString(password.toString());
    }


    // 打乱字符串顺序
    private static String shuffleString(String input) {
        List<Character> characters = new ArrayList<>();
        for (char c : input.toCharArray()) {
            characters.add(c);
        }
        Collections.shuffle(characters);
        StringBuilder output = new StringBuilder(characters.size());
        for (char c : characters) {
            output.append(c);
        }
        return output.toString();
    }

    /**
     * 删除下单商品的购物车信息
     */
    private void deleteCartItemList(List<OmsCartItemDTO> cartPromotionItemList, Long userId) {
        List<Long> ids = new ArrayList<>();
        for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
            ids.add(cartPromotionItem.getId());
        }
        omsCartItemService.delete(ids, userId);
    }

    /**
     * 计算该订单赠送的成长值
     */
    private Integer calcGiftGrowth(List<OmsOrderItemDTO> orderItemList) {
        Integer sum = 0;
        for (OmsOrderItemDTO orderItem : orderItemList) {
            sum = sum + orderItem.getGiftGrowth() * orderItem.getGoodsQuantity();
        }
        return sum;
    }

    /**
     * 计算该订单赠送的积分
     */
    private Integer calcGifIntegration(List<OmsOrderItem> orderItemList) {
        int sum = 0;
        for (OmsOrderItem orderItem : orderItemList) {
            sum += orderItem.getGiftIntegration() * orderItem.getGoodsQuantity();
        }
        return sum;
    }

    /**
     * 将优惠券信息更改为指定状态
     *
     * @param couponId  优惠券id
     * @param userId    会员id
     * @param useStatus 0->未使用；1->已使用
     */
    private void updateCouponStatus(Long couponId, Long userId, Integer useStatus) {
        if (couponId == null) return;
        //查询第一张优惠券
        UserCoupon example = this.userCouponService.getUserCouponInfo(couponId, userId);
        if (null == example) {
            return;
        }
        example.setUsed(useStatus);
        example.setStatus(UserCouponStatus.USED);
        example.setConsumptionTime(new Date());
        this.userCouponService.updateUserCouponUserd(example, couponId, userId);
    }

    private void handleRealAmount(List<OmsOrderItem> orderItemList) {
        for (OmsOrderItem orderItem : orderItemList) {
            //原价-促销优惠-优惠券抵扣-积分抵扣
            BigDecimal realAmount = orderItem.getGoodsPrice()
//                    .subtract(orderItem.getPromotionAmount())
                    .subtract(orderItem.getCouponAmount());
//                    .subtract(orderItem.getIntegrationAmount());
            orderItem.setRealAmount(realAmount);
        }
    }

    /**
     * 获取订单促销信息
     */
    private String getOrderPromotionInfo(List<OmsOrderItem> orderItemList) {
        StringBuilder sb = new StringBuilder();
        for (OmsOrderItem orderItem : orderItemList) {
            sb.append(orderItem.getPromotionName());
            sb.append(";");
        }
        String result = sb.toString();
        if (result.endsWith(";")) {
            result = result.substring(0, result.length() - 1);
        }
        return result;
    }

    /**
     * 计算订单应付金额
     */
    private BigDecimal calcPayAmount(OmsOrder order) {
        //总金额+运费-促销优惠-优惠券优惠-积分抵扣
        BigDecimal payAmount = order.getTotalAmount()
                .add(order.getFreightAmount())
                .subtract(order.getPromotionAmount())
                .subtract(order.getCouponAmount())
                .subtract(order.getIntegrationAmount());
        return payAmount;
    }

    /**
     * 计算订单优惠券金额
     */
    private BigDecimal calcIntegrationAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal integrationAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getIntegrationAmount() != null) {
                integrationAmount =
                        integrationAmount.add(orderItem.getIntegrationAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return integrationAmount;
    }

    /**
     * 计算订单优惠券金额
     */
    private BigDecimal calcCouponAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal couponAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getCouponAmount() != null) {
                couponAmount =
                        couponAmount.add(orderItem.getCouponAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return couponAmount;
    }

    /**
     * 计算订单活动优惠
     */
    private BigDecimal calcPromotionAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal promotionAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getPromotionAmount() != null) {
                promotionAmount =
                        promotionAmount.add(orderItem.getPromotionAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return promotionAmount;
    }

    /**
     * 获取可用积分抵扣金额
     *
     * @param useIntegration 使用的积分数量
     * @param totalAmount    订单总金额
     * @param currentMember  使用的用户
     * @param hasCoupon      是否已经使用优惠券
     */
//    private BigDecimal getUseIntegrationAmount(Integer useIntegration, BigDecimal totalAmount, UmsMember
//    currentMember, boolean hasCoupon) {
//        BigDecimal zeroAmount = new BigDecimal(0);
//        //判断用户是否有这么多积分
//        if (useIntegration.compareTo(currentMember.getIntegration()) > 0) {
//            return zeroAmount;
//        }
//        //根据积分使用规则判断是否可用
//        //是否可与优惠券共用
//        UmsIntegrationConsumeSetting integrationConsumeSetting = integrationConsumeSettingMapper.selectByPrimaryKey
//        (1L);
//        if (hasCoupon && integrationConsumeSetting.getCouponStatus().equals(0)) {
//            //不可与优惠券共用
//            return zeroAmount;
//        }
//        //是否达到最低使用积分门槛
//        if (useIntegration.compareTo(integrationConsumeSetting.getUseUnit()) < 0) {
//            return zeroAmount;
//        }
//        //是否超过订单抵用最高百分比
//        BigDecimal integrationAmount = new BigDecimal(useIntegration).divide(new BigDecimal
//        (integrationConsumeSetting.getUseUnit()), 2, RoundingMode.HALF_EVEN);
//        BigDecimal maxPercent = new BigDecimal(integrationConsumeSetting.getMaxPercentPerOrder()).divide(new
//        BigDecimal(100), 2, RoundingMode.HALF_EVEN);
//        if (integrationAmount.compareTo(totalAmount.multiply(maxPercent)) > 0) {
//            return zeroAmount;
//        }
//        return integrationAmount;
//    }

    /**
     * 对优惠券优惠进行处理
     *
     * @param orderItemList order_item列表
     * @param userCoupon 可用优惠券详情
     */
    /**
     * 对优惠券优惠进行处理
     *
     * @param orderItemList order_item列表
     * @param userCouponDTO 可用优惠券详情
     */
    /*private void handleCouponAmount(List<OmsOrderItem> orderItemList, CouponDTO userCouponDTO) {
        if (userCouponDTO.getType().equals(0)) {
            //全场通用
            calcPerCouponAmount(orderItemList, userCouponDTO.getCouponDTO());
        } else if (userCouponDTO.getType().equals(1)) {
            //指定分类
            List<OmsOrderItem> couponOrderItemList = getCouponOrderItemByRelation(userCouponDTO, orderItemList, 0);
            calcPerCouponAmount(couponOrderItemList, userCouponDTO.getCouponDTO());
        } else if (userCouponDTO.getType().equals(2)) {
            //指定商品
            List<OmsOrderItem> couponOrderItemList = getCouponOrderItemByRelation(userCouponDTO, orderItemList, 1);
            calcPerCouponAmount(couponOrderItemList, userCouponDTO.getCouponDTO());
        }
    }*/

    /**
     * 对优惠券优惠进行处理
     *
     * @param orderItemList order_item列表
     * @param userCoupon 可用优惠券详情
     */
    private boolean handleCouponAmount(List<OmsOrderItem> orderItemList, UserCoupon userCoupon) {
        boolean flag = false; //是否使用优惠券
        BigDecimal amount = userCoupon.getAmount();
        BigDecimal doorsillAmount = userCoupon.getDoorsillAmount();
        Long merchantId = userCoupon.getMerchantId();
        BigDecimal totalAmount = calcTotalAmount(orderItemList);
        if (CouponType.GIVE.equals(userCoupon.getType())) {
            //全场通用
            for (OmsOrderItem orderItem : orderItemList) {
                if(merchantId == 0L){
                    if(totalAmount.compareTo(amount) > 0){
                        orderItem.setCouponAmount(amount);
                        return true;
                    }
                    break;
                } else if(merchantId.equals(orderItem.getMerchantId())){
                    Map<Long, BigDecimal> map = handle(orderItemList);
                    BigDecimal merchantAmount = map.get(merchantId);
                    if(merchantAmount.compareTo(amount) > 0){
                        orderItem.setCouponAmount(amount);
                        return true;
                    } else if(merchantAmount.compareTo(amount) <= 0 && totalAmount.compareTo(amount) > 0){
                        orderItem.setCouponAmount(merchantAmount);
                        return true;
                    }
                    break;
                }
            }
        } else if (CouponType.REDUCE.equals(userCoupon.getType())) {
            //满减
            for (OmsOrderItem orderItem : orderItemList) {
                if(merchantId == 0L){
                    if(totalAmount.compareTo(amount) <= 0){
                        break;
                    }
                    if(totalAmount.compareTo(doorsillAmount) <= 0){
                        break;
                    }
                    orderItem.setCouponAmount(amount);
                    return true;
                } else if(merchantId.equals(orderItem.getMerchantId())){
                    Map<Long, BigDecimal> map = handle(orderItemList);
                    BigDecimal merchantAmount = map.get(merchantId);
                    if(merchantAmount.compareTo(doorsillAmount) <= 0){
                        break;
                    }
                    if(merchantAmount.compareTo(amount) > 0){
                        orderItem.setCouponAmount(amount);
                    } else if(merchantAmount.compareTo(amount) <= 0 && totalAmount.compareTo(amount) > 0){
                        orderItem.setCouponAmount(merchantAmount);
                    } else {
                        break;
                    }
                    return true;
                }
            }
        } else if (CouponType.DISCOUNT.equals(userCoupon.getType())) {
            //折扣
            if(amount.compareTo(BigDecimal.ZERO) <= 0 || amount.compareTo(BigDecimal.TEN) >= 0){
                return flag;
            }
            for (OmsOrderItem orderItem : orderItemList) {
                if(merchantId == 0L){
                    BigDecimal multiply = totalAmount.multiply(amount.divide(BigDecimal.TEN, 2, RoundingMode.UP));
                    orderItem.setCouponAmount(totalAmount.subtract(multiply));
                    return true;
                } else if(merchantId.equals(orderItem.getMerchantId())){
                    Map<Long, BigDecimal> map = handle(orderItemList);
                    BigDecimal merchantAmount = map.get(merchantId);
                    BigDecimal multiply = merchantAmount.multiply(amount.divide(BigDecimal.TEN, 2, RoundingMode.UP));
                    orderItem.setCouponAmount(merchantAmount.subtract(multiply));
                    return true;
                }
            }
        }
        return flag;
    }

    public Map<Long, BigDecimal> handle(List<OmsOrderItem> orderItemList){
        List<OrderPriceGroupDTO> list = orderItemList.stream()
                .map(e -> new OrderPriceGroupDTO(e.getMerchantId(),
                        e.getGoodsPrice().multiply(new BigDecimal(e.getGoodsQuantity()))))
                .collect(Collectors.toList());
        return list.stream()
                .collect(Collectors.groupingBy(OrderPriceGroupDTO::getMerchantId,
                        Collectors.reducing(BigDecimal.ZERO, OrderPriceGroupDTO::getAmount,
                                BigDecimal::add)));
    }

    /**
     * 对每个下单商品进行优惠券金额分摊的计算
     *
     * @param orderItemList 可用优惠券的下单商品商品
     */
    private void calcPerCouponAmount(List<OmsOrderItem> orderItemList, UserCoupon coupon) {
        BigDecimal totalAmount = calcTotalAmount(orderItemList);
        for (OmsOrderItem orderItem : orderItemList) {
            //(商品价格/可用商品总价)*优惠券面额
            BigDecimal couponAmount =
                    orderItem.getGoodsPrice().divide(totalAmount, 3, RoundingMode.HALF_EVEN).multiply(coupon.getAmount());
            orderItem.setCouponAmount(couponAmount);
        }
    }

    /**
     * 获取与优惠券有关系的下单商品
     *
     * @param couponHistoryDetail 优惠券详情
     * @param orderItemList       下单商品
     * @param type                使用关系类型：0->相关分类；1->指定商品
     */
    private List<OmsOrderItem> getCouponOrderItemByRelation(UserCouponDTO couponHistoryDetail,
                                                            List<OmsOrderItem> orderItemList, int type) {
        List<OmsOrderItem> result = new ArrayList<>();
        if (type == 0) {
            List<Long> categoryIdList = new ArrayList<>();
//            for (SmsCouponProductCategoryRelation productCategoryRelation : couponHistoryDetail
//            .getCategoryRelationList()) {
//                categoryIdList.add(productCategoryRelation.getProductCategoryId());
//            }
            for (OmsOrderItem orderItem : orderItemList) {
                if (categoryIdList.contains(orderItem.getGoodsCategoryId())) {
                    result.add(orderItem);
                } else {
                    orderItem.setCouponAmount(new BigDecimal(0));
                }
            }
        } else if (type == 1) {
            List<Long> productIdList = new ArrayList<>();
//            for (SmsCouponProductRelation productRelation : couponHistoryDetail.getProductRelationList()) {
//                productIdList.add(productRelation.getGoodsId());
//            }
            for (OmsOrderItem orderItem : orderItemList) {
                if (productIdList.contains(orderItem.getGoodsId())) {
                    result.add(orderItem);
                } else {
                    orderItem.setCouponAmount(new BigDecimal(0));
                }
            }
        }
        return result;
    }

    /**
     * 获取该用户可以使用的优惠券
     *
     * @param cartPromotionItemList 购物车优惠列表
     * @param couponId              使用优惠券id
     */
    private UserCouponDTO getUseCoupon(List<OmsCartItemDTO> cartPromotionItemList, Long couponId) {
//        List<UserCouponDTO> couponHistoryDetailList = memberCouponService.listCart(cartPromotionItemList, 1);
        List<UserCouponDTO> couponHistoryDetailList = new ArrayList<>();
        for (UserCouponDTO couponHistoryDetail : couponHistoryDetailList) {
            if (couponHistoryDetail.getId().equals(couponId)) {
                return couponHistoryDetail;
            }
        }
        return null;
    }

    /**
     * 计算总金额
     */
    private BigDecimal calcTotalAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal totalAmount = new BigDecimal("0");
        for (OmsOrderItem item : orderItemList) {
            totalAmount = totalAmount.add(item.getGoodsPrice().multiply(new BigDecimal(item.getGoodsQuantity())));
        }
        return totalAmount;
    }

    /**
     * 锁定下单商品的所有库存
     */
//    private void lockStock(List<CartPromotionItemDTO> cartPromotionItemList) {
//        for (CartPromotionItemDTO cartPromotionItem : cartPromotionItemList) {
//            PmsSkuStock skuStock = skuStockMapper.selectByPrimaryKey(cartPromotionItem.getGoodsSkuId());
//            skuStock.setLockStock(skuStock.getLockStock() + cartPromotionItem.getQuantity());
//            skuStockMapper.updateByPrimaryKeySelective(skuStock);
//        }
//    }

    /**
     * 判断下单商品是否都有库存
     */
    private boolean hasStock(List<CartPromotionItemDTO> cartPromotionItemList) {
        for (CartPromotionItemDTO cartPromotionItem : cartPromotionItemList) {
            if (cartPromotionItem.getRealStock() == null || cartPromotionItem.getRealStock() <= 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * 计算购物车中商品的价格
     */
    private CalcAmountDTO calcCartAmount(List<OmsCartItemDTO> cartPromotionItemList) {
        CalcAmountDTO calcAmount = new CalcAmountDTO();
        calcAmount.setFreightAmount(new BigDecimal(0));
        BigDecimal totalAmount = new BigDecimal("0");
        BigDecimal promotionAmount = new BigDecimal("0");
        for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
            totalAmount =
                    totalAmount.add(cartPromotionItem.getPrice().multiply(new BigDecimal(cartPromotionItem.getQuantity())));
//            promotionAmount = promotionAmount.add(cartPromotionItem.getReduceAmount().multiply(new BigDecimal
//            (cartPromotionItem.getQuantity())));
        }
        calcAmount.setTotalAmount(totalAmount);
        calcAmount.setPromotionAmount(promotionAmount);
        calcAmount.setPayAmount(totalAmount.subtract(promotionAmount));
        return calcAmount;
    }

    @Override
    public Map<String, Object> getOrderQuantity(Long merchantId) {
        Map<String, Object> map = this.baseMapper.getOrderQuantity(merchantId);
        if (map.get("orderCount").toString().equals("0")) {
            map.put("amount", 0);
        }
        return map;
    }

    @Override
    public Pagination<OmsOrderDTO> getMerchantOrderPaging(OrderPagingParam param) {
        Long userId = SessionContextHolder.getId();
        List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
        if (ObjectUtils.isEmpty(merchants)) {
            return null;
        }
        Page<OmsOrder> page = this.lambdaQuery()
                .eq(StringUtils.notEmpty(param.getStatus()), OmsOrder::getStatus, param.getStatus())
                .in(OmsOrder::getMerchantId, merchants)
                .eq(StringUtils.notEmpty(param.getOrderId()),OmsOrder::getId,param.getOrderId())
                .eq(StringUtils.notEmpty(param.getOrderSn()),OmsOrder::getOrderSn,param.getOrderSn())
                .orderByDesc(OmsOrder::getCreateTime)
                .page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<OmsOrderDTO> pagination = DatabaseContextHolder.pagination(page, OrderCopier.INSTANCE::copying);
        pagination.getRecords().forEach(item -> {
            item.setGoodsInfo(this.omsOrderItemService.getOrderItemByOrderId(item.getId()));
            UserDTO user = this.userService.getUser(item.getUserId());
            if (null != user){
                item.setAvatar(user.getAvatar());
                item.setMemberUsername(user.getNickname());
            }
        });
        return pagination;
    }

    @Override
    public void orderVerification(Long orderId, Long merchantId) {
        AssertUtils.nonnull(orderId, "订单不存在");
        OmsOrder omsOrder = baseMapper.selectById(orderId);
        AssertUtils.nonnull(omsOrder, "订单不存在");
        //List<Long> merchantIds = this.merchantService.getMerchantIdsByUserId(merchantUserId);
        AssertUtils.nonnull(merchantId, "商家不存在");
        AssertUtils.available(omsOrder.getMerchantId() != null && omsOrder.getMerchantId().equals(merchantId),
                "该订单商品不属于此商家");
        Integer status = omsOrder.getStatus();
        AssertUtils.available(status == 1, "该订单不可用");
        DatabaseContextHolder.transactional((Executor) () -> {
            //处理分佣逻辑
            orderHandleService.handleOrder(omsOrder);
            this.reservationService.updateStatusByOrderId(orderId, ReserveStatus.COMPLETED);
        });
    }

    @Override
    public OrderDTO queryOrderManage(Long merchantId) {
        return omsOrderMapper.queryOrderManage(merchantId);
    }

    @Override
    public MerchantTeamDTO queryTeamOrder(Long merchantId) {
        return omsOrderMapper.queryTeamOrder(merchantId);
    }


    @Override
    public BigDecimal queryAmountByLastMonth(Long merchantId) {
        LocalDateTime startLocalTime = DateTimeUtils.getLastMonthStartLocalTime();
        LocalDateTime endLocalTime = DateTimeUtils.getLastMonthEndLocalTime();
        return omsOrderMapper.queryAmountByLastMonth(merchantId, startLocalTime, endLocalTime);
    }


    @Override
    public void updateOrderStatus(Long orderId,Integer status) {
        try{
            List<Long> orderIds = new ArrayList<>();
            orderIds.add(orderId);
            this.baseMapper.updateOrderStatus(orderIds, 5);
        }catch (Exception e){
            e.getStackTrace();
        }
    }

    @Override
    public OmsOrderDetailDTO getOraderDetail(String orderSn,Long merchantId) {
        OmsOrder omsOrder = this.lambdaQuery()
                .eq(OmsOrder::getOrderSn,orderSn)
                .eq(OmsOrder::getMerchantId,merchantId)
                .eq(OmsOrder::getStatus,1)
                .one();
        if (null == omsOrder){
            return null;
        }
        List<OmsOrderItemDTO> orderItemList = omsOrderItemService.getOrderItemByOrderId(omsOrder.getId());
        OmsOrderDetailDTO orderDetail = new OmsOrderDetailDTO();
        BeanUtils.copyProperties(omsOrder, orderDetail);
        orderDetail.setOrderItemList(orderItemList);
        MerchantDTO merchant = this.merchantService.getMerchantById(omsOrder.getMerchantId());
        orderDetail.setMerchantInfo(merchant);
        return orderDetail;
    }
    @Override
    public void orderCallBacl(OrderResponse orderResponse){
        //修改订单支付状态
        OmsOrder order = this.baseMapper.selectById(orderResponse.getRequestId());
        if(order == null){
            return ;
        }
        order.setStatus(1);
        order.setPaymentTime(new Date());
        order.setPayType(1);
        DatabaseContextHolder.transactional((Executor) () ->
                orderHandleService.paySuccess(order)
        );
    }

    @Override
    public void orderNotify(SxyNotifyParam param) {
        JSONObject responseData = TradeHandle.decrypt(param);
        log.info("支付回调参数：{}", responseData);
        if(responseData == null){
            return;
        }
        String requestId = responseData.getString("requestId");
        OmsOrder omsOrder = this.baseMapper.selectById(requestId);
        if(omsOrder == null){
            //订单不存在
            return;
        }
        final Integer[] orderStatus = {omsOrder.getStatus()};
        if(orderStatus[0] != 0){
            return;
        }
        BigDecimal payAmount = omsOrder.getPayAmount();
        String orderAmount = responseData.getString("orderAmount");
        if(payAmount.multiply(new BigDecimal("100")).compareTo(new BigDecimal(orderAmount)) != 0){
            //订单金额不一样
            return;
        }
        String status = responseData.getString("status");
        String serialNumber = responseData.getString("serialNumber");
        final BigDecimal[] feeAmount = {BigDecimal.ZERO};
        DatabaseContextHolder.transactional((Executor) () -> {
            if("SUCCESS".equals(status)){
                orderStatus[0] = 1;
                String fee = responseData.getString("feeAmount");
                if(StringUtils.isBlank(fee)){
                    fee = "0";
                }
                feeAmount[0] = new BigDecimal(fee).divide(new BigDecimal("100"), 2, RoundingMode.UP);
                BigDecimal realityAmount = payAmount.subtract(feeAmount[0]);
                //平台资产变更-添加冻结金额
                BigDecimal totalAmount = assetService.updateAsset(0, realityAmount, 0L, 0, 1);
                balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L, payAmount, feeAmount[0],
                        realityAmount, totalAmount, 0, 5, omsOrder.getUserId());
            } else if("FAILED".equals(status)){
                orderStatus[0] = 8;
            } else if("CANCEL".equals(status)){
                orderStatus[0] = 7;
            }
            baseMapper.updateStatus(omsOrder.getId(), orderStatus[0], feeAmount[0], serialNumber, new Date());
            omsOrderOperateHistoryService.insertOmsOrderOperateHistory(omsOrder.getId(), omsOrder.getUserId(), omsOrder.getStatus());
        });
    }

    @Override
    public void orderRefund(OrderRefundParam param) {
        Long orderId = param.getOrderId();
        OmsOrder omsOrder = baseMapper.selectById(orderId);
        if(omsOrder == null){
            throw new ResourceUnavailableException("订单不存在");
        }
        Long userId = SessionContextHolder.getId();
        Long orderUserId = omsOrder.getUserId();
        if(userId == null || !userId.equals(orderUserId)){
            throw new ResourceUnavailableException("暂无权限");
        }
        Integer status = omsOrder.getStatus();
        if(status != 1){
            throw new ResourceUnavailableException("该订单状态不支持退款");
        }
        //查询申请中的退款订单
        int num = orderRefundService.queryApplyOrder(orderId);
        if(num > 0){
            throw new ResourceUnavailableException("该订单退款已申请");
        }
        Long refundId = orderRefundService.insert(omsOrder, param);
        onlinePayService.refund(omsOrder, refundId);
    }

    @Override
    public void orderRefundNotify(SxyNotifyParam param) {
        JSONObject responseData = TradeHandle.decrypt(param);
        log.info("退款回调参数：{}", responseData);
        if(responseData == null){
            return;
        }
        String requestId = responseData.getString("requestId");
        String amount = responseData.getString("amount");
        OmsOrderRefund refund = orderRefundService.queryById(Long.valueOf(requestId));
        Integer refundStatus = refund.getStatus();
        if(refundStatus != 1){
            return;
        }
        OmsOrder omsOrder = baseMapper.selectById(refund.getOrderId());
        BigDecimal refundAmount = refund.getRefundAmount();
        DatabaseContextHolder.transactional((Executor) () -> {
            if(refundAmount.multiply(new BigDecimal("100")).compareTo(new BigDecimal(amount)) != 0){
                //订单金额不一样
                orderRefundService.updateStatus(Long.valueOf(requestId), 3, "退款金额不一致");
                return;
            }
            String status = responseData.getString("status");
            String serialNumber = responseData.getString("serialNumber");
            BigDecimal payAmount = omsOrder.getPayAmount();
            BigDecimal feeAmount = omsOrder.getFeeAmount();
            BigDecimal realAmount = payAmount.subtract(feeAmount);
            if("SUCCESS".equals(status)){
                orderRefundService.updateStatus(Long.valueOf(requestId), 2, "", serialNumber);
                //减掉平台冻结金额
                assetService.reduceFrozenAsset(payAmount, 0L, 0);
                BigDecimal totalAmount = assetService.queryTotal(0L, 0);
                //记录余额明细
                balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L,
                        payAmount, feeAmount, realAmount, totalAmount, 1, 8, omsOrder.getUserId());
                //更改订单状态
                updateStatusById(omsOrder.getId(), 9);
                omsOrderOperateHistoryService.insertOmsOrderOperateHistory(omsOrder.getId(), omsOrder.getUserId(), omsOrder.getStatus());
            } else if("FAILED".equals(status)){
                String errorMessage = responseData.getString("errorMessage");
                orderRefundService.updateStatus(Long.valueOf(requestId), 3, errorMessage, serialNumber);
                //解除冻结
                assetService.reduceFrozenAsset(realAmount, 0L, 0);
            }
        });
    }

    @Override
    public OmsOrder queryCheckOrder(Long orderId, Long userId) {
        LambdaQueryWrapper<OmsOrder> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(OmsOrder::getId, orderId);
        wrapper.eq(OmsOrder::getUserId, userId);
        return baseMapper.selectOne(wrapper);
    }

    private int updateStatusById(Long id, Integer status) {
        UpdateWrapper<OmsOrder> update = Wrappers.update();
        update.set("status", status);
        update.eq("id", id);
        return baseMapper.update(null, update);
    }
}
